(* ::Package:: *)

(*

A Mathematica Package for 2Vect and Modular Tensor Categories

Daniel A. Roberts
Department of Computer Science
University of Oxford

MTCategories.m

*)

BeginPackage["MTCategories`", {"TwoVect`"}]

GenerateMonoidalStructures::usage =
		"GenerateMonoidalStructures[m,u,DisplayForm] generates a complete set of monoidal category 2-Cell structures, \[Alpha], \[Rho], and \[Lambda] (with the correct 1-Cell source and destination), for a given product functor 1-Cell m and unit 1-Cell u. The third argument, DisplayForm, is optional (and is set to False as a default) and will display the structures using TwoCellForm (as opposed to the standard form which can be used for input or solving)."

GenerateMTCStructures::usage =
				"GenerateMTCStructures[m,u,DisplayForm] generates a complete set of modular tensor category 2-Cell structures (including the full set of monoidal category structures), \[Alpha], \[Rho],  \[Lambda], \[Beta], and \[Theta] (with the correct 1-Cell source and destination), for a given product functor 1-Cell m and unit 1-Cell u. The third argument, DisplayForm, is optional (and is set to False as a default) and will display the structures using TwoCellForm (as opposed to the standard form which can be used for input or solving)."

GenerateHandle::usage =
		"GenerateHandle[m,u] generates the handle (which is a 1-Cell object) of the MTC given by product functor m and unit u. The handle is the sum over all the simple objects of that simple object fused with its dual: \[CapitalSigma] A (x) A*, for all simple objects A."

IsPentagon::usage =
		"IsPentagon[m,\[Alpha],WithErrors,DisplayForm] is a function that calculates and tests the pentagon equation, an axiom of monoidal categories, where the 1-Cell m specifies the fusion rules, and the 2-Cell \[Alpha] specifies the associator. The last two arguments are optional (and default to True and True, respectively). IsPentagon[m,\[Alpha]] or IsPentagon[m,\[Alpha],True] tests the form and consistency of m and \[Alpha], whether \[Alpha] is invertible, and the pentagon equation itself, and then will specify any errors. Pentagon[m,\[Alpha],False] or Pentagon[m,\[Alpha],False,True] prints the axiom in display form. Pentagon[m,\[Alpha],False,False] returns the equation in input form, which can then be given to a solver. IsPentagon[m,\[Alpha],False,DisplayForm] is essentially intended to be used when \[Alpha] has variable entries. Pentagon[m,\[Alpha],False,False] essentially is the same function as IsPentagon[m,\[Alpha],True], except without all the checks for form and consistency."

IsTriangle::usage =
		"IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,WithErrors,DisplayForm]  is a function that calculates and tests the triangle equation, an axiom of monoidal categories, where the 1-Cell m specifies the fusion rules, the 1-Cell u specifies the unit, the 2-Cell \[Rho] specifies the right unitor, and the 2-Cell \[Lambda] specifies the left unitor, and the 2-Cell \[Alpha] specifies the associator. The last two arguments are optional (and default to True and True, respectively). IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u] or IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,True] tests the form and consistency of the arguments, whether \[Rho] and \[Lambda] are invertible, and the triangle equation itself, and then will specify any errors. IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,False] or IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,False,True] prints the axiom in display form. IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,False,False] returns the equation in input form, which can then be given to a solver. IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,False,DisplayForm] is essentially intended to be used when \[Rho], \[Lambda], and/or \[Alpha] have variable entries. IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,False,False] essentially is the same function as IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u,True], except without all the checks for form and consistency. Note: this function should only be called after the category is shown to satisfy the pentagon equation."

IsMonoidal::usage =
		"IsMonoidal[m,\[Alpha],\[Rho],\[Lambda],u] tests if the input structures form a monodial category, where m is the 1-Cell tensor product functor specifying the fusion rules, u is the 1-Cell unit, \[Alpha] is the 2-Cell associator, \[Rho] is the 2-Cell right unitor, and \[Lambda] is the 2-Cell left unitor."

IsBraided::usage =
		"IsBraided[m,\[Alpha],\[Beta]] tests if a monoidal category is Braided with the input 2-Cell \[Beta]. For this function to be used, m and \[Alpha] (and other structures, of course) must be shown already to form a monoidal category."

IsHexagon1::usage =
		"IsHexagon1[m,\[Alpha],\[Beta],DisplayForm] is a function that calculates if a monoidal category satisfies the first hexagon equation for braided monoidal categories (an axiom of MTCs), where the 1-Cell m specifies the fusion rules, the 2-Cell \[Alpha] specifies the associator, and the 2-Cell \[Beta] specifies the braid. The last argument is optional (and defaults to True). IsHexagon1[m,\[Alpha],\[Beta]] or IsHexagon1[m,\[Alpha],\[Beta],True] prints the axiom in display form. IsHexagon1[m,\[Alpha],\[Beta],False] returns the equation in input form, which can then be given to a solver. IsHexagon1[m,\[Alpha],\[Beta],DisplayForm] is essentially intended to be used when \[Alpha] and/or \[Beta] have variable entries."

IsHexagon2::usage =
		"IsHexagon2[m,\[Alpha],\[Beta],DisplayForm] is a function that calculates if a monoidal category satisfies the second hexagon equation for braided monoidal categories (an axiom of MTCs), where the 1-Cell m specifies the fusion rules, the 2-Cell \[Alpha] specifies the associator, and the 2-Cell \[Beta] specifies the braid. The last argument is optional (and defaults to True). IsHexagon2[m,\[Alpha],\[Beta]] or IsHexagon2[m,\[Alpha],\[Beta],True] prints the axiom in display form. IsHexagon2[m,\[Alpha],\[Beta],False] returns the equation in input form, which can then be given to a solver. IsHexagon2[m,\[Alpha],\[Beta],DisplayForm] is essentially intended to be used when \[Beta] has variable entries, but MUST NOT be used when \[Alpha] has variable entries since \[Alpha] inverse is required."

IsTwisted::usage =
		"IsTwisted[m,\[Beta],\[Theta],WithErrors,DisplayForm] is a function that calculates if a braided monoidal category has a Twist (an axiom of MTCs), where the 1-Cell m specifies the fusion rules, the 2-Cell \[Beta] specifies the braid, and the 2-Cell \[Theta] specifies the twist. The last two arguments are optional (and default to True and True, respectively). IsTwisted[m,\[Beta],\[Theta]] or IsTwisted[m,\[Beta],\[Theta],True] tests the form and consistency of the arguments, whether \[Theta] is invertible, and the twist equation itself, and then will specify any errors. IsTwisted[m,\[Beta],\[Theta],False] or IsTwisted[m,\[Beta],\[Theta],False,True] prints the axiom in display form. IsTwisted[m,\[Beta],\[Theta],False,False] returns the equation in input form, which can then be given to a solver. IsTwisted[m,\[Beta],\[Theta],False,DisplayForm] is essentially intended to be used when \[Theta] and/or \[Beta] have variable entries. IsTwisted[m,\[Beta],\[Theta],False,False] essentially is the same function as IsTwisted[m,\[Beta],\[Theta],True], except without all the checks for form and consistency. Note: this function should only be called after m and \[Beta] (with additional structures, of course) are shown form a braided monoidal category."

IsFrobenius::usage =
		"IsFrobenius[m] tests the Frobenius condition, which is necessary but not sufficient for confirming the rigidity property of MTCs."

GenerateDual::usage =
		"GenerateDual[x,m,u] returns the dual candidate of the 1-Cell x (i.e. tentatively x*). Note: this can only be called if IsFrobenius[m] is True."

IsRigid::usage =
		"IsRigid[m,\[Alpha],u] tests if a monoidal category satisfies the rigidity property (i.e. has duals). For this function to be used, m and \[Alpha] must form a monoidal category."

IsRibbonCoherent::usage =
		"IsRibbonCoherent[m,\[Theta],u,DisplayForm] tests whether the 2-Cell twist \[Theta] is compatible with duals. Essentially, this tests whether the twisted braided monoidal category is a ribbon category. The last argument is optional (and defaults to FALSE). IsRibbonCoherent[m,\[Theta],u] or IsRibbonCoherent[m,\[Theta],u,False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. IsRibbonCoherent[m,\[Theta],u,True] prints the equation in display form. For this function to be used, m, \[Theta], and u (with additional structures, of course) must form a twisted braided monoidal category, and IsRigid[m,\[Alpha],u] must be True."

IsModular::usage =
		"IsModular[m,u,\[Beta]] tests if a braided twisted monoidal category that is rigid satisfies the modular property, where m is the 1-Cell fusion rules, u is the 1-Cell unit, and \[Beta] is the 2-Cell braid."

IsMTC::usage =
		"IsMTC[m,\[Alpha],\[Rho],\[Lambda],u,\[Beta],\[Theta]] tests if the input structures form a modular tensor category, where  where m is the 1-Cell product functor, \[Alpha] is the 2-Cell associator, \[Rho] is the 2-Cell right unitor, \[Lambda] is the 2-Cell left unitor, u is the 1-Cell unit, \[Beta] is the 2-Cell braid, and \[Theta] is the 2-Cell twist."

IsMonoidAssociative::usage = 
		"IsMonoidAssociative[m,A,\[Mu],\[Alpha],DisplayForm] determines whether the 2-Cell multiplication \[Mu] satisfies the monoid associativity equation for the 1-Cell A to be a monoid. The last argument is optional (and defaults to True). IsMonoidAssociative[m,A,\[Mu],\[Alpha]] or IsMonoidAssociative[m,A,\[Mu],\[Alpha],True] prints the equation in display form. IsMonoidAssociative[m,A,\[Mu],\[Alpha],False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. For this function to be used, the 1-Cell fusion rules m and the 2-Cell associator \[Alpha] (with additional structures, of course) must form a monoidal category."

IsMonoidRightUnit::usage = 
		"IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho],DisplayForm] determines whether the 2-Cell multiplication \[Mu] and the 2-Cell unit \[Eta] satisfies the monoid right unit equation for \[Eta] to be the unit of the 1-Cell monoid A. The last argument is optional (and defaults to True). IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho]] or IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho],True] prints the equation in display form. IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho],False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. For this function to be used, the 1-Cell fusion rules m, the 1-Cell unit u, the 2-Cell associator \[Alpha], and the 2-Cell right unitor \[Rho] (with additional structures, of course) must form a monoidal category."

IsMonoidLeftUnit::usage = 
		"IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda],DisplayForm] determines whether the 2-Cell multiplication \[Mu] and the 2-Cell unit \[Eta] satisfies the monoid left unit equation for \[Eta] to be the unit of the 1-Cell monoid A. The last argument is optional (and defaults to True). IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda]] or IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda],True] prints the equation in display form. IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda],False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. For this function to be used, the 1-Cell fusion rules m, the 1-Cell unit u, the 2-Cell associator \[Alpha], and the 2-Cell left unitor \[Lambda] (with additional structures, of course) must form a monoidal category."

IsMonoid::usage =
		"IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] determines whether A, \[Mu], and \[Eta] form a monoid (i.e. an algebra). A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell multiplication, and \[Eta] is the 2-Cell unit. The structures m, u, \[Alpha], \[Rho], \[Lambda] must form a monoidal category with m the 1-Cell product functor, u the 1-Cell unit, \[Alpha] the 2-Cell associator, \[Rho] the 2-Cell right unitor, and \[Lambda] the 2-Cell left unitor. Note: does not specify the error if the function returns False."

IsMonoidDaggerFrobenius::usage =
		"IsMonoidDaggerFrobenius[m,\[Alpha],A,\[Mu],\[Eta],DisplayForm] determines whether the monoid given by A, \[Mu], and \[Eta] is Dagger-Frobenius. The last argument is optional (and defaults to FALSE). IsMonoidDaggerFrobenius[m,\[Alpha],A,\[Mu],\[Eta]] or IsMonoidDaggerFrobenius[m,\[Alpha],A,\[Mu],\[Eta],False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. IsMonoidDaggerFrobenius[m,\[Alpha],A,\[Mu],\[Eta],True] prints the equation in display form. For this function to be used, the 1-Cell fusion rules m and the 2-Cell associator \[Alpha] (with additional structures, of course) must form a monoidal category, and IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] must be True (i.e. A, \[Mu], and \[Eta] must already be shown to form a monoid)."

IsMonoidCommutative::usage =
		"IsMonoidCommutative[m,A,\[Mu],\[Beta],DisplayForm] determines whether the monoid given by A and \[Mu] is Commutative. The last argument is optional (and defaults to FALSE). IsMonoidCommutative[m,A,\[Mu],\[Beta]] or IsMonoidCommutative[m,A,\[Mu],\[Beta],False] returns the equation in input form, which (if there are variable entries) can then be given to a solver. IsMonoidCommutative[m,A,\[Mu],\[Beta],True] prints the equation in display form. For this function to be used, the 1-Cell fusion rules m, the 2-Cell associator \[Alpha], and the 2-Cell braid \[Beta] (with additional structures, of course) must form a braided monoidal category, and IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] must be True (i.e. A, \[Mu], and \[Eta] must already be shown to form a monoid)."

IsMonoidalFunctorStructureEquation1::usage =
		"IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,WithErrors,DisplayForm] is a function that tests if the 1-Cell functor F and 2-Cell structure \[Phi] (and the structures of the two monoidal categories, labeled by subscripts 1 and 2, respectively) satsify the first structure equation for F to be a strong monoidal functor. This equation shows that [(FA (.) FB) (.) FC] goes to [F ( A (x) (B (x) C) )] in two different ways, where here (x) is used to represent m1 and (.) is is used to represent m2. The last two arguments are optional (and default to True and True, respectively). IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2] or IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,True] tests the form and consistency of the arguments, whether \[Phi] is invertible, and the structure equation itself, and then will specify any errors. IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,False] or IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,False,True] prints the axiom in display form. IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,False,False] returns the equation in input form, which can then be given to a solver. IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,False,DisplayForm] is essentially intended to be used when \[Phi] has variable entries. IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,False,False] essentially is the same function as IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2,True], except without all the checks for form and consistency. Note: this function should only after the categories given by suscripts 1 and 2 in the input are shown form monoidal categories."

IsMonoidalFunctorStructureEquation2and3::usage =
		"IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,WithErrors,DisplayForm] is a function that the 1-Cell functor F and 2-Cell structures \[Phi]u and \[Phi] (and the structures of the two monoidal categories, labeled by subscripts 1 and 2, respectively) satsify the second and third structure equations for F to be a strong monoidal functor. The second equation shows that [FA (.) u2] goes to [FA] in two different ways, and the third equation shows that [u2 (.) FB] goes to [FB] in two different ways, where here (x) is used to represent m1 and (.) is is used to represent m2. The last two arguments are optional (and default to True and True, respectively). IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2] or IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,True] tests the form and consistency of the arguments, whether \[Phi]u is invertible, and the structure equation itself, and then will specify any errors. IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,False] or IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,False,True] prints the axiom in display form. IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,False,False] returns a 1D list of all the individual equations from the two 2-matrix equations, which can then be given to a solver. IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,False,DisplayForm] is essentially intended to be used when \[Phi]u has variable entries. IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,False,False] essentially is the same function as IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2,True], except without all the checks for form and consistency. Note: this function should only after the categories given by suscripts 1 and 2 in the input are shown form monoidal categories and the first monoidal structure equation is shown to be satisfied."

IsStrongMonoidalFunctor::usage =
		"IsStrongMonoidalFunctor[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2] tests if proposed structures (the 1-Cell F and the 2-Cells \[Phi] and \[Phi]u) form a strong monoidal functor from the category given by product functor m1 to the category given by product functor m2. The structures labeled by 1 are from the category given by m1 and structures labaled by 2 are from the category given by m2."

SolveMonoidal::usage =
		"SolveMonoidal[m,u] solves for the 2-Cells \[Alpha], \[Lambda], and \[Rho], for the candidate monoidal category with product functor 1-Cell m and unit 1-Cell u. This function is a prototype and tends to run out of memory."

\[Alpha]ABC::usage =
		"\[Alpha]ABC[m,\[Alpha],A,B,C] returns the 2-Cell given by \[Alpha]_{A,B,C} (i.e. \[Alpha] applied to objects A, B, and C), where A, B, and C are 1-Cell objects of the monoidal category, and m is the 1-Cell product functor. The output 2-Cell is automatically appropriately reboxed."

\[Beta]AB::usage =
		"\[Beta]AB[m,\[Beta],A,B] returns the 2-Cell given by \[Beta]_{A,B} (i.e. \[Beta] applied to objects A and B), where A and B are 1-Cell objects of the braided monoidal category, and m is the 1-Cell product functor. The output 2-Cell is automatically appropriately reboxed."

\[Theta]A::usage =
		"\[Theta]A[m,\[Theta],A] returns the 2-Cell given by \[Theta]_A (i.e. \[Theta] applied to object A), where A is a 1-Cell object of the twisted braided monoidal category, and m is the 1-Cell product functor. The output 2-Cell is automatically appropriately reboxed."

\[Lambda]A::usage =
		"\[Lambda]A[m,\[Lambda],u,A] returns the 2-Cell given by \[Lambda]_A (i.e. \[Lambda] applied to object A), where A is a 1-Cell object of the monoidal category, m is the 1-Cell product functor, and u is the 1-Cell unit object. The output 2-Cell is automatically appropriately reboxed."

\[Rho]A::usage =
		"\[Rho]A[m,\[Rho],u,A] returns the 2-Cell given by \[Rho]_A (i.e. \[Rho] applied to object A), where A is a 1-Cell object of the monoidal category, m is the 1-Cell product functor, and u is the 1-Cell unit object. The output 2-Cell is automatically appropriately reboxed."

\[Phi]AB::usage =
		"\[Phi]AB[m1,m2,F,\[Phi],A,B] returns the 2-Cell given by \[Phi]_{A,B} (i.e. \[Phi] applied to objects A and B), where A and B are 1-Cell objects of the monoidal category given by m1. F is the 1-Cell monoidal functor, m1 is the 1-Cell product functor of the first category, and m2 is the 1-Cell product functor of the second category. The output 2-Cell is automatically appropriately reboxed."


(*Begin["`Private`"]*)


(*Generate Functions*)

(*Monoidal Categories*)

(* Return the source of \[Alpha], the associator, from input of m, the monodial functor *)
Generate\[Alpha]Source[m_]:=OneCellComposition[m,OneCellTimes[m,GetIdentityOneCell[GetDest[m]]]]

(* Return the destination of \[Alpha], the associator, from input of m, the monodial functor *)
Generate\[Alpha]Dest[m_]:=OneCellComposition[m,OneCellTimes[GetIdentityOneCell[GetDest[m] ],m] ]

(* return source of \[Rho] from m and u, the unit *)
Generate\[Rho]Source[m_,u_]:=OneCellComposition[m,OneCellTimes[GetIdentityOneCell[GetDest[m]],u]]

(* return Destination, which is the identity OneCell *)
Generate\[Rho]Dest[m_,u_]:=GetIdentityOneCell[GetDest[m]]

(* return source of \[Rho] from m and u, the unit *)
Generate\[Lambda]Source[m_,u_]:=OneCellComposition[m,OneCellTimes[u,GetIdentityOneCell[GetDest[m]]]]

(* return domain, which is the identity OneCell *)
Generate\[Lambda]Dest[m_,u_]:=GetIdentityOneCell[GetDest[m]]

(* Returns a completely variable \[Alpha] (with the correct OneCell source and destination) for a given m *)
GenerateVariable\[Alpha][m_]:=GenerateVariableTwoCell[Generate\[Alpha]Source[m],Generate\[Alpha]Dest[m]]

(* Returns a completely variable \[Rho] (with the correct OneCell source and destination) for a given m and u *)
GenerateVariable\[Rho][m_,u_]:=GenerateVariableTwoCell[Generate\[Rho]Source[m,u],Generate\[Rho]Dest[m,u]]

(* Returns a completely variable \[Lambda] (with the correct OneCell source and destination) for a given m and u*)
GenerateVariable\[Lambda][m_,u_]:=GenerateVariableTwoCell[Generate\[Lambda]Source[m,u],Generate\[Lambda]Dest[m,u]]

(* Returns a complete set of the monoidal structures, \[Alpha], \[Rho], and \[Lambda] (with the correct OneCell source and destination), 
for a given m and u. Has a third optional argument for the output in display form (TwoCellForm)
 as opposed to a form which can be used for input or solving. *)
GenerateMonoidalStructures[m_,u_,DisplayForm_:False]:=Module[{},
	If[DisplayForm,
		Print["\[Alpha] = ",GenerateVariable\[Alpha][m]//TwoCellForm];
		Print["\[Rho] = ",GenerateVariable\[Rho][m,u]//TwoCellForm];
		Print["\[Lambda] = ",GenerateVariable\[Lambda][m,u]//TwoCellForm];
		,
		Print["\[Alpha] = ",GenerateVariable\[Alpha][m]];
		Print["\[Rho] = ",GenerateVariable\[Rho][m,u]];
		Print["\[Lambda] = ",GenerateVariable\[Lambda][m,u]];
	]
]

(*Modular Tensor Categories*)

(* Returns the source of \[Beta], the braid transformation, from input of m, the monodial functor *)
Generate\[Beta]Source[m_]:=m

(* Returns the destination of \[Beta], the braid transformation, from input of m, the monodial functor *)
Generate\[Beta]Dest[m_]:=OneCellComposition[m,GetOneCellSwap[GetDest[m],GetDest[m]]]

(* Returns the source of \[Theta], the twist transformation, from input of m, the monodial functor 
(really, since \[Theta] goes from id to id, it uses m to find out what the base object is of the monoidal category and the details of m don't matter) *)
Get\[Theta]Source[m_]:=GetIdentityOneCell[GetDest[m]]

(* Returns the destination of \[Theta], the twist transformation, from input of m, the monodial functor 
(really, since \[Theta] goes from id to id, it uses m to find out what the base object is of the monoidal category and the details of m don't matter) *)
Get\[Theta]Dest[m_]:=GetIdentityOneCell[GetDest[m]]

(* Returns a completely variable \[Beta] (with the correct OneCell source and destination) for a given m *)
GenerateVariable\[Beta][m_]:=GenerateVariableTwoCell[Generate\[Beta]Source[m],Generate\[Beta]Dest[m]]

(* Returns a completely variable \[Theta] (with the correct OneCell source and destination) for a given m. 
Note: this one is actually trivial as it is the identity TwoCell on the based category*)
GetVariable\[Theta][m_]:=GenerateVariableTwoCell[Get\[Theta]Source[m],Get\[Theta]Dest[m]]

(* Returns a complete set of the modular tensor structures (including the full set of monoidal structures), \[Alpha], \[Rho],  \[Lambda], \[Beta], and \[Theta] 
(with the correct OneCell source and destination), for a given m and u. Has a third optional argument for the output in display form 
(TwoCellForm) as opposed to a form which can be used for input or solving. *)
GenerateMTCStructures[m_,u_,DisplayForm_:False]:=Module[{},
	GenerateMonoidalStructures[m,u,DisplayForm];
	If[DisplayForm,
		Print["\[Beta] = ",GenerateVariable\[Beta][m]//TwoCellForm];
		Print["\[Theta] = ",GetVariable\[Theta][m]//TwoCellForm];
		,
		Print["\[Beta] = ",GenerateVariable\[Beta][m]];
		Print["\[Theta] = ",GetVariable\[Theta][m]];
	]
]

(*Monoids (i.e. Algebras)*)

(*Generates the Handle of an MTC*)
GenerateHandle[m_,u_]:=
Module[{n,i,x,xstar,handle},
	n=GetDest[m];
	handle=ConstantArray[0,{n,1}];

	For[i=1,i<=n,i++,

		x=ConstantArray[0,{n,1}];
		x[[i,1]]=1;
		x=OneCell[x];

		xstar=GenerateDual[x,m,u];
		handle=handle+(MonoidalTensorProduct[m,x,xstar]//OneToMat);

	];

	handle//OneCell
]

(*Monoidal Functors*)

(* Return the source of \[Phi], the coherence natural transformation, from input of m2, the monodial functor of D, and F the monoidal functor between C and D *)
Generate\[Phi]Source[F_,m2_]:=OneCellComposition[m2,OneCellTimes[F,F]]

(* Return the destination of \[Phi], the coherence natural transformation, from input of m1, the monodial functor of C, and F the monoidal functor between C and D *)
Generate\[Phi]Dest[F_,m1_]:=OneCellComposition[F,m1]

(* Return the source of \[Phi]u, the coherence morphism associated with the units, from the input of u2, the unit of D *)
Generate\[Phi]uSource[u2_]:=u2

(* Return the destination of \[Phi]u, the coherence morphism associated with the units, from the input of u1, the unit of C, and F, the monoidal functor between C and D *)
Generate\[Phi]uDest[F_,u1_]:=OneCellComposition[F,u1]

(* Returns a completely variable \[Phi] (with the correct OneCell source and destination) for a given F, m1, and m2 *)
GenerateVariable\[Phi][F_,m1_,m2_]:=GenerateVariableTwoCell[Generate\[Phi]Source[F,m2],Generate\[Phi]Dest[F,m1]]

(* Returns a completely variable \[Phi]u (with the correct OneCell source and destination) for a given F, u1, and u2 *)
GenerateVariable\[Phi]u[F_,u1_,u2_]:=GenerateVariableTwoCell[Generate\[Phi]uSource[u2],Generate\[Phi]uDest[F,u1]]

(* Returns a complete set of the structures for F, a monoidal functor (\[Phi] and \[Phi]u), with the correct OneCell source and destination, 
for a given F going from category 1 with m1 and u1 to category 2 with m2 and u2. Has a third optional argument for the output in display form 
(TwoCellForm) as opposed to a form which can be used for input or solving. *)
GenerateMonoidalFunctorStructures[F_,m1_,u1_,m2_,u2_,DisplayForm_:False]:=Module[{},
	If[DisplayForm,
		Print["\[Phi] = ",GenerateVariable\[Phi][F,m1,m2]//TwoCellForm];
		Print["\[Phi]u = ",GenerateVariable\[Phi]u[F,u1,u2]//TwoCellForm];
		,
		Print["\[Phi] = ",GenerateVariable\[Phi][F,m1,m2]];
		Print["\[Phi]u = ",GenerateVariable\[Phi]u[F,u1,u2]];
	]
]


(*Monoidal Category Equations*)

(* test whether m is a valid monoidal functor in the sense is [(A (x) B) (x) C] the same functor as [(A (x) (B (x) C)] *)
IsValidM[m_]:=Generate\[Alpha]Source[m]==Generate\[Alpha]Dest[m]

(* One of the two ways to rebracket [((A (x) B) (x) C)) (x) D] to [A (x) (B (x) (C (x) D))] 
where m is the monodial operation and \[Alpha] is the associator. 
The function returns the rebracketing TwoCell.
Note: I suspsect that all of the \[Tau] restructures in this specific case will be the identity 
(due to the majority of the OneCells being combined are identities). 
However, I have included them for completeness since I have a function that generates \[Tau] (just as I also made note of where \[Psi] would go - 
but not actually including it since \[Psi] is identity for 2Vect and including a specific operation would be generating an identity via the identity function and then applying it) *)
Rebracket1[m_,\[Alpha]_]:=Module[{\[Alpha]id,Leg1,id,ABxCD,Leg2,id\[Alpha],Leg3,\[Tau]1,\[Tau]2, \[Tau]1m,m\[Tau]2Inv,\[Tau]Leg1,\[Omega]1,\[Omega]2,\[Omega]2Inv,\[Omega]\[Tau]Leg1,\[Omega]Leg2,\[Tau]2Inv,\[Tau]\[Omega]Leg2},

	id=GetIdentityOneCell[GetDest[m]];

	(* apply first \[Alpha] to A, B, and C *)
	\[Alpha]id=TwoCellTimes[\[Alpha],GetIdentityTwoCell[id]];
	Leg1=WhiskerRight[m,\[Alpha]id];

	(* apply \[Tau] structure to rebox for next \[Alpha] *)
	\[Tau]1 = Get\[Tau][m,
		OneCellTimes[id,m],
		id,
		id
	];

	\[Tau]1m = WhiskerRight[m,\[Tau]1];
	\[Tau]Leg1 = TwoCellVerticalComposition[\[Tau]1m,Leg1];

	(* apply \[Omega] structure to continue reboxing for next \[Alpha] *)
	\[Omega]1= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[id,m],id]
	];
	\[Omega]\[Tau]Leg1=TwoCellVerticalComposition[\[Omega]1,\[Tau]Leg1];

	(* Leg1 complete. Now build up the next leg, which applies \[Alpha] to A, (BxC), and D. *)
	ABxCD=OneCellTimes[OneCellTimes[id, m], id];
	Leg2=WhiskerLeft[\[Alpha],ABxCD];

	(* now rebox by essentially undoing the previous reboxing, first by applying inverse \[Omega] *)
	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[OneCellTimes[id,m],id]
	];
	\[Omega]2Inv=TwoCellInverse[\[Omega]2];
	\[Omega]Leg2= TwoCellVerticalComposition[\[Omega]2Inv,Leg2];

	(* Now, if there needed to be a restructure due to the non-associativity of monoidal multiplication (which may be noted \[Psi] in various places), 
	 it would go here to change (idm [x] m) [x] idm to idm [x] (m [x] idm). However, this is trivial twice over since  (a) the tensor product is associative, 
	 in 2Vect the monoidal 2 operation is associative and (b) two of the three inputs are the identity. So then we continue with the reboxing applying a \[Tau] inverse *)
	\[Tau]2 = Get\[Tau][id,
		id,
		m,
		OneCellTimes[m,id]
	];
	\[Tau]2Inv = TwoCellInverse[\[Tau]2];
	m\[Tau]2Inv=WhiskerRight[m,\[Tau]2Inv];
	\[Tau]\[Omega]Leg2= TwoCellVerticalComposition[m\[Tau]2Inv,\[Omega]Leg2];

	(* Leg 2 complete. Now build up final leg, which applies \[Alpha] to B,C, and D. *)
	id\[Alpha]=TwoCellTimes[GetIdentityTwoCell[id],\[Alpha]];
	Leg3=WhiskerRight[m,id\[Alpha]];

	(*Combine all the legs*)
	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[\[Tau]\[Omega]Leg2,\[Omega]\[Tau]Leg1]]
]

(* The other way to rebracket [((A (x) B) (x) C)) (x) D] to [A (x) (B (x) (C (x) D))] 
where m is the monodial operation and \[Alpha] is the associator.
Note (same as above): I suspsect that all of the \[Tau] restructures in this specific case will be the identity 
(due to the majority of the OneCells being combined are identities). 
However, I have included them for completeness since I have a function that generates \[Tau] (just as I also made note of where \[Psi] would go - 
but not actually including it since \[Psi] is identity for 2Vect and including a specific operation would be generating an identity via the identity function and then applying it) *)
Rebracket2[m_,\[Alpha]_]:=Module[{AxBCD,Leg1,ABCxD,Leg2,id, idx2,\[Tau]1,m\[Tau]1,\[Omega]1,Leg1\[Omega]\[Tau],\[Omega]2,\[Omega]2Inv,\[Tau]2 ,\[Tau]2Inv,Leg2\[Omega],\[Omega]3,\[Omega]3Inv,m\[Tau]2Inv,\[Tau]\[Omega],\[Omega]2andhalf,Inv\[Tau]Extra,\[Tau]Extra,extrabox},

	id=GetIdentityOneCell[GetDest[m]];
	idx2=GetIdentityOneCell[GetSource[m]];

	(*first rebox, since the assumed start form allowed Rebracket1 to apply its \[Alpha] right away *)

	(*apply \[Tau] which will probably always be identity *)
	\[Tau]1 = Get\[Tau][m,
		OneCellTimes[m,id],
		id,
		id
	];

	m\[Tau]1=WhiskerRight[m,\[Tau]1];

	(*then apply \[Omega] to finally allow for the first \[Alpha] *)
	\[Omega]1= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[m,id],id]
	];

	(*apply \[Alpha] to AxB, C, and D *)
	AxBCD=OneCellTimes[m,idx2];
	Leg1=WhiskerLeft[\[Alpha],AxBCD];

	(*completing Leg1, we get: *)
	Leg1\[Omega]\[Tau]= TwoCellVerticalComposition[Leg1,TwoCellVerticalComposition[\[Omega]1,m\[Tau]1]];



	(* Rebox again to get ready to apply \[Alpha] again  *)
	(* apply \[Omega] inverse *)
	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[m,OneCellTimes[id,id]]
	];
	\[Omega]2Inv=TwoCellInverse[\[Omega]2];

	(* Now we would apply \[Psi] to rebracket (m [x] idm) [x] idm to m [x] (idm [x] idm). But this will always be identity for 2Vect and because of the 2 identities *) 

	(* Now we could apply \[Tau] inverse, followed by moving the parallel m's up and down in parallel (which is only a pictorial operation), followed by reapplying \[Tau]. However, that would just amount to applying \[Tau] inverse followed by \[Tau] or the identity. *)
	(*I changed my mind, I am adding it back in because they are different \[Tau]s*)

	Inv\[Tau]Extra = TwoCellInverse[Get\[Tau][id,
		m,
		m,
		OneCellTimes[id,id]
	]];

	\[Tau]Extra = Get\[Tau][m,
		OneCellTimes[id,id],
		id,
		m
	];

	extrabox=TwoCellHorizontalComposition[GetIdentityTwoCell[m],TwoCellVerticalComposition[\[Tau]Extra,Inv\[Tau]Extra]];

	\[Omega]2andhalf= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[id,id],m]
	];

	(* Compute Leg 2 *)
	ABCxD=OneCellTimes[idx2,m];
	Leg2=WhiskerLeft[\[Alpha],ABCxD];

	(*combine with the previous \[Omega] and \[Omega] Inverse (which are in fact different \[Omega] so they don't equal the identity)*)
	Leg2\[Omega] = TwoCellVerticalComposition[Leg2,TwoCellVerticalComposition[\[Omega]2andhalf,TwoCellVerticalComposition[extrabox,\[Omega]2Inv]]];

	(*Leg2 complete. Now add the final reboxing so that this rebracket can actually be compared to the other rebracket *)

	(* If \[Psi] != id, then it would be applied here to convert ((idm [x] idm) m) to (idm [x] (idm [x] m)) which would again be rendered trivial for dual reasons. *)

	(* apply \[Omega] inverse *)
	\[Omega]3= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[id,OneCellTimes[id,m]]
	];
	\[Omega]3Inv=TwoCellInverse[\[Omega]3];

	(* Finally, apply \[Tau] Inverse *)
	\[Tau]2 = Get\[Tau][id,
		id,
		m,
		OneCellTimes[id,m]
	];
	\[Tau]2Inv=TwoCellInverse[\[Tau]2];
	m\[Tau]2Inv=WhiskerRight[m,\[Tau]2Inv];

	(*combine final reboxing *)
	\[Tau]\[Omega]=TwoCellVerticalComposition[m\[Tau]2Inv,\[Omega]3Inv];

	(* combine the 2 legs and the final reboxing *)

	TwoCellVerticalComposition[\[Tau]\[Omega], TwoCellVerticalComposition[Leg2\[Omega],Leg1\[Omega]\[Tau]]]
]

(* Tests if a category satisfies the pentagon equation with associator \[Alpha] as part of the test of a monoidal category. *)
IsPentagon[m_,\[Alpha]_,WithErrors_:True,DisplayForm_:True]:=Module[{\[Alpha]SourceFromM,\[Alpha]DestFromM},
	If[WithErrors,

		\[Alpha]SourceFromM=Generate\[Alpha]Source[m];
		\[Alpha]DestFromM=Generate\[Alpha]Dest[m];

		If[\[Alpha]SourceFromM!=\[Alpha]DestFromM,
			Print["Error: m can never be a product functor for a monoidal category. [(A (x) B) (x) C] does not give the same functor as [(A (x) (B (x) C)]."];
			False,
			If[\[Alpha]SourceFromM!=GetSource[\[Alpha]]||\[Alpha]DestFromM!=GetDest[\[Alpha]],
				Print["Error: m and \[Alpha] are incompatible. The functor given by [((A (x) B) (x) C) (x) D] is not the same as source and destination OneCell of \[Alpha]."];
				False,

				If[!IsTwoCellInvertible[\[Alpha]],
					Print["Error: \[Alpha] is not invertible and thus is not an isomorphism."];
					False,

					If[FullSimplify[Rebracket1[m,\[Alpha]]==Rebracket2[m,\[Alpha]]],
						True,
						Print["m and \[Alpha] are compatible but do not satisfy the Pentagon equation."];
						False
					]
				]
			]
		]
	,

		If[DisplayForm,
			CellByCellTwoMatrixEquation[FullSimplify[Rebracket1[m,\[Alpha]]]//TwoToTwoMat,FullSimplify[Rebracket2[m,\[Alpha]]]//TwoToTwoMat]//TwoMatrixForm
			,
			FullSimplify[TwoToTwoMat[Rebracket1[m,\[Alpha]]]==TwoToTwoMat[Rebracket2[m,\[Alpha]]]]
		]
	]
]

(* First path in the triangle equation, which goes from [(A (x) I) (x) B] to [A (x) B].
 Note: as for the Pentagon equations, its not clear which \[Tau] or \[Omega] operations are always going to be Identity, 
 and they are all included for completeness *)
TrianglePathA[m_,\[Rho]_]:=Module[{id,\[Rho]id},
	id=GetIdentityOneCell[GetDest[m]];
	\[Rho]id=TwoCellTimes[\[Rho],GetIdentityTwoCell[id]];
	WhiskerRight[m, \[Rho]id]
]

(* Second path in the triangle equation, which goes from [(A (x) I) (x) B] to [A (x) B]. 
Note: as for the Pentagon equations, its not clear which \[Tau] or \[Omega] operations are always going to be Identity, 
and they are all included for completeness *)
TrianglePathB[m_,\[Alpha]_,\[Lambda]_,u_]:=Module[{id,id\[Lambda],\[Tau]1,m\[Tau]1,\[Omega]1,Leg2,iduid,Leg2\[Omega]\[Tau],\[Omega]2,\[Omega]2Inv,\[Tau]2,\[Tau]2Inv,m\[Tau]2Inv,Leg3,Leg3\[Tau]\[Omega]},

	id=GetIdentityOneCell[GetDest[m]];
	iduid=OneCellTimes[OneCellTimes[id,u],id];

	(*Compute the second two sides of the triangle equation using \[Lambda] and a lot of reboxing. Start by reboxing, since we assumed a start state that allowed for application of \[Rho] and not \[Alpha]. *)

	(*apply \[Tau]*)
	\[Tau]1 = Get\[Tau][m,
		OneCellTimes[id,u],
		id,
		id
	];
	m\[Tau]1=WhiskerRight[m, \[Tau]1];

	(*apply \[Omega]*)
	\[Omega]1= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[id,u],id]
	];

	(*now apply \[Alpha] to rebracket *)
	Leg2=WhiskerLeft[\[Alpha],iduid];
	Leg2\[Omega]\[Tau]=TwoCellVerticalComposition[Leg2,TwoCellVerticalComposition[\[Omega]1,m\[Tau]1]];

	(*Now compute the third and final leg. But first, rebox!*)

	(*first, if \[Psi] wasn't id and if this wasn't trivial anyway due to the two id's, we would apply \[Psi] to change ((id [x] u) [x] id) to (id [x] (u [x] id)) *)

	(*apply \[Omega] inverse *)
	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[id,OneCellTimes[u,id]]
	];
	\[Omega]2Inv = TwoCellInverse[\[Omega]2];

	(*apply \[Tau] inverse *)
	\[Tau]2 = Get\[Tau][id,
		id,
		m,
		OneCellTimes[u,id]
	];
	\[Tau]2Inv=TwoCellInverse[\[Tau]2];
	m\[Tau]2Inv=WhiskerRight[m,\[Tau]2Inv];

	(*finally, apply \[Lambda]*)
	id\[Lambda]=TwoCellTimes[GetIdentityTwoCell[id],\[Lambda]];
	Leg3=WhiskerRight[m,id\[Lambda]];
	Leg3\[Tau]\[Omega]=TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[m\[Tau]2Inv,\[Omega]2Inv]];

	TwoCellVerticalComposition[Leg3\[Tau]\[Omega],Leg2\[Omega]\[Tau]]
]

(* Tests if the category satisfies the traingle equation as part of the test of a monoidal category.*)
(* Note: there is an order to which the errors happen instead of giving all of the possible ones that apply. 
NOTE: this does not test for incompatible m and \[Alpha], because the should only be called AFTER category is shown 
to satisfy the pentagon equation with assocaitor \[Alpha].  *)
IsTriangle[m_,\[Alpha]_,\[Rho]_,\[Lambda]_,u_,WithErrors_:True,DisplayForm_:True]:=Module[{\[Lambda]Source,\[Rho]Source},
	If[WithErrors,
		\[Rho]Source=Generate\[Rho]Source[m,u];
		\[Lambda]Source=Generate\[Lambda]Source[m,u];

		If[\[Rho]Source!=Generate\[Rho]Dest[m,u],
			Print["Error: Using m and u, (A (x) I) is not the same OneCell as the identity on (A)."];
			False,

			If[\[Lambda]Source!=Generate\[Lambda]Dest[m,u],
				Print["Error: Using m and u, (I (x) A) is not the same OneCell as the identity on (A)."];
				False,

				If[\[Rho]Source!=GetSource[\[Rho]],
					Print["Error: m and u, and \[Rho] are incompatible. The functor given by (A (x) I) is not the same as the source and destination OneCell of \[Rho]."];
					False,

					If[\[Lambda]Source!=GetSource[\[Lambda]],
						Print["Error: m and u, and \[Lambda] are incompatible. The functor given by (I (x) A) is not the same as the source and destination OneCell of \[Lambda]."];
						False,

						If[!IsTwoCellInvertible[\[Rho]],
							Print["Error: \[Rho] is not invertible and thus is not an isomorphism."];
							False,

							If[!IsTwoCellInvertible[\[Lambda]],
								Print["Error: \[Lambda] is not invertible and thus is not an isomorphism."];
								False,

								If[FullSimplify[TrianglePathA[m,\[Rho]]==TrianglePathB[m,\[Alpha],\[Lambda],u]],
									True,
									Print["m, \[Alpha], \[Rho], \[Lambda], and u are all compatible but they do not satisfy the Triangle equation."];
									False
								]
							]
						]
					]
				]
			]
		]
		,
		If[DisplayForm,
			CellByCellTwoMatrixEquation[FullSimplify[TrianglePathA[m,\[Rho]]]//TwoToTwoMat,FullSimplify[TrianglePathB[m,\[Alpha],\[Lambda],u]]//TwoToTwoMat]//TwoMatrixForm
			,
			FullSimplify[TwoToTwoMat[TrianglePathA[m,\[Rho]]]==TwoToTwoMat[TrianglePathB[m,\[Alpha],\[Lambda],u]]]
		]
	]
]

(* Tests if proposed structures form a monodial category, where m is the tensor product functor, 
\[Alpha] is the associator, \[Rho] is the right unitor, \[Lambda] is the left unitor, and u is the unit. *)
(* Note: There is an order to which the errors happen instead of giving all of the possible ones that apply. 
But this makes sense, as the pentagon equation needs to be shown to be satisfied first before the function that tests the triangle equation is called *)
IsMonoidal[m_,\[Alpha]_,\[Rho]_,\[Lambda]_,u_]:=Module[{},

	If[!IsPentagon[m,\[Alpha]],
		Print["Pentagon equation is not satisfied."];
		False,

		If[!IsTriangle[m,\[Alpha],\[Rho],\[Lambda],u],
			Print["Pentagon equation is satisfied but the triangle equation is not."];
			False,
			True
		]
	]
]


(*Modular Tensor Category Equations*)

(* First to go from (A (x) B) (x) C to B (x) (C (x) A), which is the top of the first Hexagon equation *)
Hexagon1Top[m_,\[Alpha]_,\[Beta]_]:=Module[{id,n,Snn,Snnxn,Leg1,Leg2,Leg3,\[Omega]1,\[Omega]1Inv,\[Sigma]1,idm\[Sigma]1,\[Omega]2,\[Omega]\[Sigma]\[Omega]Leg2},

	n=GetDest[m];
	id=GetIdentityOneCell[n];
	Snn=GetOneCellSwap[n,n];
	Snnxn = GetOneCellSwap[n,n*n];

	(* compute the first leg *)
	Leg1 = \[Alpha];

	(* compute the second leg and subsequent reboxing *)
	Leg2 =WhiskerLeft[\[Beta],OneCellTimes[id,m]];

	(*apply \[Omega] inverse*)
	\[Omega]1= Get\[Omega][m,
		Snn,
		OneCellTimes[id,m]
	];
	\[Omega]1Inv = TwoCellInverse[\[Omega]1];

	(*apply \[Sigma] rewrite*)
	\[Sigma]1=Get\[Sigma][id,m];
	idm\[Sigma]1=WhiskerRight[m,\[Sigma]1];

	(*apply \[Omega]*)
	\[Omega]2=Get\[Omega][m,
		OneCellTimes[m,id],
		Snnxn
	];

	\[Omega]\[Sigma]\[Omega]Leg2=TwoCellVerticalComposition[\[Omega]2,TwoCellVerticalComposition[idm\[Sigma]1,TwoCellVerticalComposition[\[Omega]1Inv,Leg2]]];

	(* final leg *)
	Leg3=WhiskerLeft[\[Alpha],Snnxn];

	(* Complete Path*)
	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[\[Omega]\[Sigma]\[Omega]Leg2,Leg1]]
]

(* Second to go from (A (x) B) (x) C to B (x) (C (x) A), which is the bottom of the first Hexagon equation *)
Hexagon1Bottom[m_,\[Alpha]_,\[Beta]_]:=Module[{id,n,Snn,Snnxn,Leg1,Leg2,Leg3,\[Tau]1,idm\[Tau]1,\[Omega]1,\[Omega]\[Tau]Leg1,\[Omega]2,\[Omega]2Inv,\[Tau]2,idm\[Tau]2idS,\[Omega]3,\[Omega]3Inv,idm\[Omega]3Inv,\[Omega]4,\[Omega]\[Omega]\[Tau]\[Omega]Leg3},

	n=GetDest[m];
	id=GetIdentityOneCell[n];
	Snn=GetOneCellSwap[n,n];
	Snnxn = GetOneCellSwap[n,n*n];

	(* compute the first leg and rebox*)
	Leg1 =WhiskerRight[m,TwoCellTimes[\[Beta],GetIdentityTwoCell[id]]];

	(* apply \[Tau] rebox *)
	\[Tau]1 = Get\[Tau][m,
		Snn,
		id,
		id
	];
	idm\[Tau]1=WhiskerRight[m,\[Tau]1];

	(*apply \[Omega] rebox*)
	\[Omega]1= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[Snn,id]
	];

	\[Omega]\[Tau]Leg1=TwoCellVerticalComposition[\[Omega]1,TwoCellVerticalComposition[idm\[Tau]1,Leg1]];

	(* compute second leg *)
	Leg2=WhiskerLeft[\[Alpha],OneCellTimes[Snn,id]];

	(* compute third leg and rebox *)

	(*NOTE: this MAY BE dependent on a boxing order at the TwoCell Level *)
	Leg3=WhiskerLeft[WhiskerRight[m, TwoCellTimes[GetIdentityTwoCell[id],\[Beta]]],OneCellTimes[Snn,id]];

	(*apply \[Omega] inverse*)
	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,OneCellComposition[m,Snn]],
		OneCellTimes[Snn,id]
	];
	\[Omega]2Inv = TwoCellInverse[\[Omega]2];

	(* apply \[Tau] rebox *)
	\[Tau]2 = Get\[Tau][id,
		id,
		m,
		Snn
	];

	(*NOTE: this is another candidate for a boxing order at the TwoCell Level*)
	idm\[Tau]2idS=WhiskerRight[m,WhiskerLeft[\[Tau]2,OneCellTimes[Snn,id]]];

	(*apply \[Omega] inverse*)
	\[Omega]3= Get\[Omega][OneCellTimes[id,m],
		OneCellTimes[id,Snn],
		OneCellTimes[Snn,id]
	];
	\[Omega]3Inv = TwoCellInverse[\[Omega]3];
	idm\[Omega]3Inv=WhiskerRight[m,\[Omega]3Inv];

	(*apply \[Omega] rebox *)
	\[Omega]4= Get\[Omega][m,
		OneCellTimes[id,m],
		Snnxn
	];

	\[Omega]\[Omega]\[Tau]\[Omega]Leg3=TwoCellVerticalComposition[\[Omega]4,TwoCellVerticalComposition[idm\[Omega]3Inv,TwoCellVerticalComposition[idm\[Tau]2idS,TwoCellVerticalComposition[\[Omega]2Inv,Leg3]]]];

	(* Complete Path*)
	TwoCellVerticalComposition[\[Omega]\[Omega]\[Tau]\[Omega]Leg3,TwoCellVerticalComposition[Leg2,\[Omega]\[Tau]Leg1]]
]

(* First to go from A (x) (B (x) C) to (C (x) A) (x) B, which is the top of the second Hexagon equation *)
Hexagon2Top[m_,\[Alpha]_,\[Beta]_]:=Module[{id,n,Snn,Snxnn,\[Alpha]Inv,Leg1,Leg2,Leg3,\[Omega]1,\[Omega]1Inv,\[Sigma]1,idm\[Sigma]1,\[Omega]2,\[Omega]\[Sigma]\[Omega]Leg2},

	n=GetDest[m];
	id=GetIdentityOneCell[n];
	Snn=GetOneCellSwap[n,n];
	Snxnn = GetOneCellSwap[n*n,n];

	\[Alpha]Inv = TwoCellInverse[\[Alpha]];

	(* compute the first leg *)
	Leg1 = \[Alpha]Inv;

	(* compute the second leg and subsequent reboxing *)
	Leg2 =WhiskerLeft[\[Beta],OneCellTimes[m,id]];

	(*apply \[Omega] inverse*)
	\[Omega]1= Get\[Omega][m,
		Snn,
		OneCellTimes[m,id]
	];
	\[Omega]1Inv = TwoCellInverse[\[Omega]1];

	(*apply \[Sigma] rewrite*)
	\[Sigma]1=Get\[Sigma][m,id];
	idm\[Sigma]1=WhiskerRight[m,\[Sigma]1];

	(*apply \[Omega]*)
	\[Omega]2=Get\[Omega][m,
		OneCellTimes[id,m],
		Snxnn
	];

	\[Omega]\[Sigma]\[Omega]Leg2=TwoCellVerticalComposition[\[Omega]2,TwoCellVerticalComposition[idm\[Sigma]1,TwoCellVerticalComposition[\[Omega]1Inv,Leg2]]];

	(* final leg *)
	Leg3=WhiskerLeft[\[Alpha]Inv,Snxnn];

	(* Complete Path*)
	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[\[Omega]\[Sigma]\[Omega]Leg2,Leg1]]

]

(* Second to go from A (x) (B (x) C) to (C (x) A) (x) B, which is the bottom of the second Hexagon equation *)
Hexagon2Bottom[m_,\[Alpha]_,\[Beta]_]:=Module[{id,n,Snn,Snxnn,\[Alpha]Inv,Leg1,Leg2,Leg3,\[Tau]1,idm\[Tau]1,\[Omega]1,\[Omega]\[Tau]Leg1,\[Omega]2,\[Omega]2Inv,\[Tau]2,idm\[Tau]2idS,\[Omega]3,\[Omega]3Inv,idm\[Omega]3Inv,\[Omega]4,\[Omega]\[Omega]\[Tau]\[Omega]Leg3},

	n=GetDest[m];
	id=GetIdentityOneCell[n];
	Snn=GetOneCellSwap[n,n];
	Snxnn = GetOneCellSwap[n*n,n];

	\[Alpha]Inv = TwoCellInverse[\[Alpha]];

	(* compute the first leg and rebox*)
	Leg1 =WhiskerRight[m,TwoCellTimes[GetIdentityTwoCell[id] ,\[Beta] ]  ];

	(* apply \[Tau] rebox *)
	\[Tau]1 = Get\[Tau][id,
		id,
		m,
		Snn
	];
	idm\[Tau]1=WhiskerRight[m,\[Tau]1];

	(*apply \[Omega] rebox*)
	\[Omega]1= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[id,Snn]
	];

	\[Omega]\[Tau]Leg1=TwoCellVerticalComposition[\[Omega]1,TwoCellVerticalComposition[idm\[Tau]1,Leg1]];

	(* compute second leg *)
	Leg2=WhiskerLeft[\[Alpha]Inv,OneCellTimes[id,Snn]];

	(* compute third leg and rebox *)

	(*NOTE: this MAY BE dependent on a boxing order at the TwoCell Level *)
	Leg3=WhiskerLeft[WhiskerRight[m, TwoCellTimes[\[Beta],GetIdentityTwoCell[id]]],OneCellTimes[id,Snn]];

	(*apply \[Omega] inverse*)
	\[Omega]2= Get\[Omega][m,
		OneCellTimes[OneCellComposition[m,Snn],id],
		OneCellTimes[id,Snn]
	];
	\[Omega]2Inv = TwoCellInverse[\[Omega]2];

	(* apply \[Tau] rebox *)
	\[Tau]2 = Get\[Tau][m,
		Snn,
		id,
		id
	];

	(*NOTE: this is another candidate for a boxing order at the TwoCell Level*)
	idm\[Tau]2idS=WhiskerRight[m,WhiskerLeft[\[Tau]2,OneCellTimes[id,Snn]]];

	(*apply \[Omega] inverse*)
	\[Omega]3= Get\[Omega][OneCellTimes[m,id],
		OneCellTimes[Snn,id],
		OneCellTimes[id,Snn]
	];
	\[Omega]3Inv = TwoCellInverse[\[Omega]3];
	idm\[Omega]3Inv=WhiskerRight[m,\[Omega]3Inv];

	(*apply \[Omega] rebox *)
	\[Omega]4= Get\[Omega][m,
		OneCellTimes[m,id],
		Snxnn
	];

	\[Omega]\[Omega]\[Tau]\[Omega]Leg3=TwoCellVerticalComposition[\[Omega]4,TwoCellVerticalComposition[idm\[Omega]3Inv,TwoCellVerticalComposition[idm\[Tau]2idS,TwoCellVerticalComposition[\[Omega]2Inv,Leg3]]]];

	(* Complete Path*)
	TwoCellVerticalComposition[\[Omega]\[Omega]\[Tau]\[Omega]Leg3,TwoCellVerticalComposition[Leg2,\[Omega]\[Tau]Leg1]]
]

(* Tests if a Monoidal Category is Braided with a given \[Beta]. Category ought to be shown to be Monoidal first before this function is used *)
IsBraided[m_,\[Alpha]_,\[Beta]_]:=Module[{\[Beta]SourceFromM, \[Beta]DestFromM},
	\[Beta]SourceFromM=Generate\[Beta]Source[m];
	\[Beta]DestFromM=Generate\[Beta]Dest[m];

	If[\[Beta]SourceFromM!=\[Beta]DestFromM,
		Print["Error: m can never be a product functor for a braided monoidal category. [A (x) B] does not give the same functor as [B (x) A]"];
		False,

		If[\[Beta]SourceFromM!=GetSource[\[Beta]]||\[Beta]DestFromM!=GetDest[\[Beta]],
			Print["Error: m and \[Beta] are incompatible. The functor given by [A (x) B] is not the same as source and destination OneCell of \[Beta]."];
			False,

			If[!IsTwoCellInvertible[\[Beta]],
				Print["Error: \[Beta] is not invertible and thus is not an isomorphism."];
				False,

				If[FullSimplify[Hexagon1Top[m,\[Alpha],\[Beta]]!=Hexagon1Bottom[m,\[Alpha],\[Beta]]],
					Print["Error: m and \[Beta] are compatible but do not satisfy the first Hexagon equation."];
					False,

					If[FullSimplify[Hexagon2Top[m,\[Alpha],\[Beta]]!=Hexagon2Bottom[m,\[Alpha],\[Beta]]],
						Print["Error: m and \[Beta] are compatible but do not satisfy the second Hexagon equation."];
						False,
						True
					]
				]
			]
		]
	]
]

(*returns the first hexagon equation for braided monoidal categories*)
IsHexagon1[m_,\[Alpha]_,\[Beta]_,DisplayForm_:True]:=Module[{},
	If[DisplayForm,
		CellByCellTwoMatrixEquation[FullSimplify[Hexagon1Top[m,\[Alpha],\[Beta]]]//TwoToTwoMat,FullSimplify[Hexagon1Bottom[m,\[Alpha],\[Beta]]]//TwoToTwoMat]//TwoMatrixForm
			,
		FullSimplify[TwoToTwoMat[Hexagon1Top[m,\[Alpha],\[Beta]]]==TwoToTwoMat[Hexagon1Bottom[m,\[Alpha],\[Beta]]]]
	]
]

(*returns the first hexagon equation for braided monoidal categories*)
IsHexagon2[m_,\[Alpha]_,\[Beta]_,DisplayForm_:True]:=Module[{},
	If[DisplayForm,
		CellByCellTwoMatrixEquation[FullSimplify[Hexagon2Top[m,\[Alpha],\[Beta]]]//TwoToTwoMat,FullSimplify[Hexagon2Bottom[m,\[Alpha],\[Beta]]]//TwoToTwoMat]//TwoMatrixForm
		,
		FullSimplify[TwoToTwoMat[Hexagon2Top[m,\[Alpha],\[Beta]]]==TwoToTwoMat[Hexagon2Bottom[m,\[Alpha],\[Beta]]]]
	]
]

(* Top part of the twist equation which goes from A(x)B to A(x)B (i.e. by putting a twist in the ribbon) *)
TwistEquationTop[m_,\[Beta]_,\[Theta]_]:=WhiskerLeft[\[Theta],m]

(* Bottom part of the twist equation which goes from A(x)B to A(x)B (essentially by braiding it twice) *)
TwistEquationBottom[m_,\[Beta]_,\[Theta]_]:= Module[{Snn,Leg1,Leg2,Leg3},

	Snn=GetOneCellSwap[GetDest[m],GetDest[m]];

	Leg1= WhiskerRight[m,TwoCellTimes[\[Theta],\[Theta]]];
	Leg2=\[Beta];
	Leg3 = WhiskerLeft[\[Beta],Snn];

	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[Leg2,Leg1]]
]

(* Tests if a Monoidal Category has a Twist with a given Braid, \[Beta], and Twist, \[Theta]. Category ought to be shown to be Braided Monoidal first before this function is used *)
IsTwisted[m_,\[Beta]_,\[Theta]_,WithErrors_:True,DisplayForm_:True]:=Module[{\[Theta]SourceandDestFromM},
	If[WithErrors,

		(* since Get\[Theta]Source and Get\[Theta]Dest are defined as exactly the same function, we don't have to test if they are equal *)
		\[Theta]SourceandDestFromM=Get\[Theta]Source[m];

		If[\[Theta]SourceandDestFromM!=GetSource[\[Theta]]||\[Theta]SourceandDestFromM!=GetDest[\[Theta]],
			Print["Error: m and \[Theta] are incompatible. The identity OneCell is not the same as source and destination OneCell of \[Beta] (which it ought to be)."];
			False,

			If[!IsTwoCellInvertible[\[Theta]],
				Print["Error: \[Theta] is not invertible and thus is not an isomorphism."];
				False,

				If[FullSimplify[TwistEquationTop[m,\[Beta],\[Theta]]!=TwistEquationBottom[m,\[Beta],\[Theta]]],
					Print["Error: m and \[Theta] are compatible but do not satisfy the Twist equation."];
					False,
					True
				]
			]
		]
		,
		If[DisplayForm,
			CellByCellTwoMatrixEquation[FullSimplify[TwistEquationTop[m,\[Beta],\[Theta]]]//TwoToTwoMat,FullSimplify[TwistEquationBottom[m,\[Beta],\[Theta]]]//TwoToTwoMat]//TwoMatrixForm
			,
			FullSimplify[TwoToTwoMat[TwistEquationTop[m,\[Beta],\[Theta]]]==TwoToTwoMat[TwistEquationBottom[m,\[Beta],\[Theta]]]]
		]
	]
]

(* Test the Frobenius condition, which is necessary but not sufficient for confirming the rigidity property of MTCs *)
IsFrobenius[m_]:=Module[{id,mt},
	id=GetIdentityOneCell[GetDest[m]];
	mt=GetAdjointOneCell[m];
	FullSimplify[OneCellComposition[OneCellTimes[id,m],OneCellTimes[mt,id]]==OneCellComposition[OneCellTimes[m,id],OneCellTimes[id,mt]]]
]

(* Returns the dual candidate to x given the OneCell name of x (i.e. a map from 1->GetDest[m] thats isomorphic to the object.) *)
(* NOTE: This can only be used if IsFrobenius returns true. *)
GenerateDual[x_,m_,u_]:=Module[{id, mt,xt},
	id=GetIdentityOneCell[GetDest[m]];
	mt=GetAdjointOneCell[m];
	xt=GetAdjointOneCell[x];

	OneCellComposition[OneCellTimes[id,xt],OneCellComposition[mt,u]]
]

(* Tests whether the monoidal category is rigid (essentially, whether it has a well-defined sense of duals) *)
IsRigid[m_,\[Alpha]_,u_]:=Module[{n,i,uAsn,\[Alpha]Inv,rigid=True,a,astar,r,c,istar,sequence,\[Alpha]aXastarXaToa,\[Alpha]InvastarXaXastarToaStar,r1,r2,c1,c2,aastarFuse,astaraFuse,zeroesBefore,zeroesAfter,j,Basisaastar,Basisastara},

	If[IsFrobenius[m],
		n=GetDest[m];
		sequence={Array[1 #1 &,n]};

		\[Alpha]Inv=TwoCellInverse[\[Alpha]];

		uAsn=(sequence.OneToMat[u])[[1,1]];

		For[i=1,i<=n,i++,
			If[i!=uAsn,

				a=ConstantArray[0,{n,1}];
				a[[i,1]]=1;
				a=OneCell[a];

				astar=GenerateDual[a,m,u];

				(*To find the correct inner matrix of \[Alpha] and of \[Alpha]^-1, we also need to find out which element is the dual *)
				istar = (sequence.OneToMat[astar])[[1,1]];

				(*next we need to find the right basis vectors for the correct matrices *)
				aastarFuse=OneToMat[OneCellComposition[m,OneCellTimes[a,astar]]];
				zeroesBefore=0;
				zeroesAfter=0;
				For[j=1,j<=n,j++,
					If[j<uAsn&&aastarFuse[[j,1]]==0,zeroesBefore++];
					If[j>uAsn&&aastarFuse[[j,1]]==0,zeroesAfter++];
				];
				Basisaastar=ConstantArray[0,{n-zeroesBefore-zeroesAfter,1}];
				Basisaastar[[uAsn-zeroesBefore,1]]=1;

				astaraFuse=OneToMat[OneCellComposition[m,OneCellTimes[astar,a]]];
				zeroesBefore=0;
				zeroesAfter=0;
				
				For[j=1,j<=n,j++,
					If[j<uAsn&&astaraFuse[[j,1]]==0,zeroesBefore++];
					If[j>uAsn&&astaraFuse[[j,1]]==0,zeroesAfter++];
				];
				Basisastara=ConstantArray[0,{n-zeroesBefore-zeroesAfter,1}];
				Basisastara[[uAsn-zeroesBefore,1]]=1;

				(*test both rigidity conditions *)

				(*first*)
				c1=(i-1)*n^2+(istar-1)*n+i;
				r1=i;
				\[Alpha]aXastarXaToa = TwoToTwoMat[\[Alpha]][[r1,c1]];

				If[FullSimplify[(Transpose[Basisastara].(\[Alpha]aXastarXaToa.Basisaastar))[[1,1]]==0],rigid=False,,rigid=False];

				(*second*)
				c2=(istar-1)*n^2+(i-1)*n+istar;
				r2=istar;
				\[Alpha]InvastarXaXastarToaStar = TwoToTwoMat[\[Alpha]Inv][[r2,c2]];
				If[FullSimplify[(Transpose[Basisaastar].(\[Alpha]InvastarXaXastarToaStar.Basisastara))[[1,1]]==0],rigid=False,,rigid=False];
			]
		];
		rigid
		,
		(* does not satisfy the Frobenius condition, so cannot be rigid *)
		Print["Does not satisfy the Frobenius condition, which is a necessary condition for being rigid."];
		False
	]
]

(* Tests whether the twist is compatible with the duals (essentially, whether the twisted braided monoidal category is a ribbon category) *)
IsRibbonCoherent[m_,\[Theta]_,u_,DisplayForm_:False]:=
Module[{idm,idu,idid,idmdagger,PathA,PathB},

	(idm=GetIdentityTwoCell[m]);
	(idmdagger=GetIdentityTwoCell[GetAdjointOneCell[m]]);
	(idu=GetIdentityTwoCell[u]);
	(idid=GetIdentityTwoCell[GetIdentityOneCell[GetDest[m]]]);

	PathA=TwoCellHorizontalComposition[
		TwoCellTimes[\[Theta],idid],
		TwoCellHorizontalComposition[idmdagger,idu]
	];

	PathB=TwoCellHorizontalComposition[
		TwoCellTimes[idid,\[Theta]],
		TwoCellHorizontalComposition[idmdagger,idu]
	];

	If[DisplayForm,
		CellByCellTwoMatrixEquation[FullSimplify[PathA]//TwoToTwoMat,FullSimplify[PathB]//TwoToTwoMat]//TwoMatrixForm
		,
		FullSimplify[TwoToTwoMat[PathA]==TwoToTwoMat[PathB]]
	]
]

(* Test for Modularity using the second version (i.e. braiding two objects twice is NOT the identity for any two objects that aren't Direct Sums of the Identity). *) 
IsModular[m_,u_,\[Beta]_]:=Module[{n,sequence,uAsn,DoubleBraidTwoCell,i,j,c,column,row,matrixOfInterest,notIdCount,isModular=True},

	n=GetDest[m];
	sequence={Array[1 #1 &,n]};
	uAsn=(sequence.OneToMat[u])[[1,1]];

	DoubleBraidTwoCell=TwoCellVerticalComposition[WhiskerLeft[\[Beta],GetOneCellSwap[n,n]],\[Beta]];

	For[i=1,i<=n,i++,
		If[i!=uAsn,
			notIdCount=0;
			For[j=1,j<=n,j++,
				If[j!=uAsn,
					column=(i-1)*n+j;

					For[row=1,row<=n,row++,
					
						(*which is guaranteed to be square since DoubleBraidTwoCell goes from m->m*)
						matrixOfInterest=TwoToTwoMat[DoubleBraidTwoCell][[row,column]] ;
						If[FullSimplify[IdentityMat[Dim[matrixOfInterest][[1]]]!=matrixOfInterest&&!Is0D[matrixOfInterest]], notIdCount++];
					]
				]
			];
			
			If[notIdCount==0,isModular=False];
		]
	];

	isModular
]

(* Tests if proposed structure forms a Modular Tensor Category, where m is the product functor, \[Alpha] is the associator, 
\[Rho] is the right unitor, \[Lambda] is the left unitor, u is the unit (the structures needed for a Monoidal Category), \[Beta] is the braid, and \[Theta] is the twist. *)
(* Note: This DOES test whether the category is monoidal first, so that does not need to be done seperately. *)
(* Note2: There is an order to which the errors happen instead of giving all of the possible ones that apply. 
But this makes sense as the category must be monoidal before it can be braided monoidal, it must be braided monoidal before it can have a twist, 
and it must be braided before it can be modular (and the testing order among those that have the same requirements is arbitrary) *)
IsMTC[m_,\[Alpha]_,\[Rho]_,\[Lambda]_,u_,\[Beta]_,\[Theta]_]:=Module[{},

	If[!IsMonoidal[m,\[Alpha],\[Rho],\[Lambda],u],
		Print["Category is not monoidal."];
		False,

		If[!IsRigid[m,\[Alpha],u],
			Print["Category is monoidal but not rigid."];
			False,

			If[!IsBraided[m,\[Alpha],\[Beta]],
				Print["Monoidal category is rigid but not braided."];
				False,

				If[!IsTwisted[m,\[Beta],\[Theta]],
					Print["Braided monoidal category is rigid but does not have a twist."];
					False,

					If[!IsRibbonCoherent[m,\[Theta],u],
						Print["Twisted braided monoidal category's twist is not compatible with the rigid structure."];
						False,

						If[!IsModular[m,u,\[Beta]],
							Print["Braided monoidal category with a twist is rigid but not modular."];
							False,
							True
						]
					]
				]
			]
		]
	]
]


(*Monoid (i.e. Algebra) and Frobenius Algebra Equations *)

(* Fuses A and B together using the fusion rules given by m *)
MonoidalTensorProduct[m_,A_,B_]:=OneCellComposition[m,OneCellTimes[A,B]]

(*associative equation path 1 in testing whether A forms a monoid*)
Algebra\[Mu]Path1[m_,A_,\[Mu]_]:=
Module[{idA,idm,Leg1},
	(idA=GetIdentityTwoCell[A]);
	(idm=GetIdentityTwoCell[m]);
	(Leg1=TwoCellHorizontalComposition[idm,TwoCellTimes[\[Mu],idA]]);
	TwoCellVerticalComposition[\[Mu],Leg1]
]

(*associative equation path 2 in testing whether A forms a monoid*)
Algebra\[Mu]Path2[m_,A_,\[Mu]_,\[Alpha]_]:=
Module[{id,idA, idm,\[Tau]1,LegA,LegB,LegC,\[Omega]2,LegD,\[Tau]2,LegE,LegF,LegG},
	id=GetIdentityOneCell[GetDest[m]];
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];

	\[Tau]1 = Get\[Tau][m,
		OneCellTimes[A,A],
		id,
		A
	];

	LegA=TwoCellHorizontalComposition[idm,\[Tau]1 ];

	LegB= Get\[Omega][m,
			OneCellTimes[m,id],
			OneCellTimes[OneCellTimes[A,A],A]
		];

	LegC=TwoCellHorizontalComposition[\[Alpha],TwoCellTimes[TwoCellTimes[idA,idA],idA]];

	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[OneCellTimes[A,A],A]
	];

	(LegD=TwoCellInverse[\[Omega]2]);

	\[Tau]2 = Get\[Tau][id,
		A,
		m,
		OneCellTimes[A,A]
	];

	LegE=TwoCellHorizontalComposition[idm, TwoCellInverse[\[Tau]2]];

	LegF=TwoCellHorizontalComposition[idm,TwoCellTimes[idA,\[Mu]]];

	LegG=\[Mu];

	TwoCellVerticalComposition[LegG,TwoCellVerticalComposition[LegF,TwoCellVerticalComposition[LegE,TwoCellVerticalComposition[LegD,TwoCellVerticalComposition[LegC,TwoCellVerticalComposition[LegB,LegA]]]]]]
]

(*left unitor equation path 1 in testing whether A forms a monoid*)
Algebra\[Eta]LeftPath1[m_,A_,\[Mu]_,\[Eta]_]:=
Module[{idA,idm,Leg1,Leg2},
	(idA=GetIdentityTwoCell[A]);
	(idm=GetIdentityTwoCell[m]);

	Leg1=TwoCellHorizontalComposition[idm,TwoCellTimes[\[Eta],idA]];
	Leg2=\[Mu];

	TwoCellVerticalComposition[Leg2,Leg1]
]

(*left unitor equation path 2 in testing whether A forms a monoid*)
Algebra\[Eta]LeftPath2[m_,A_,\[Mu]_,u_,\[Lambda]_]:=
Module[{idA,idm,id,\[Omega]1},
	idA=GetIdentityTwoCell[A];
	idm=GetIdentityTwoCell[m];
	id=GetIdentityOneCell[GetDest[m]];

	\[Omega]1= Get\[Omega][m,
		OneCellTimes[u,id],
		A
	];

	TwoCellVerticalComposition[TwoCellHorizontalComposition[\[Lambda],idA],\[Omega]1]
]

(*right unitor equation path 1 in testing whether A forms a monoid*)
Algebra\[Eta]RightPath1[m_,A_,\[Mu]_,\[Eta]_]:=
Module[{idA,idm,Leg1,Leg2},
	(idA=GetIdentityTwoCell[A]);
	(idm=GetIdentityTwoCell[m]);

	Leg1=TwoCellHorizontalComposition[idm,TwoCellTimes[idA,\[Eta]]];
	Leg2=\[Mu];

	TwoCellVerticalComposition[Leg2,Leg1]
]

(*right unitor equation path 2 in testing whether A forms a monoid*)
Algebra\[Eta]RightPath2[m_,A_,\[Mu]_,u_,\[Rho]_]:=
Module[{idA,idm,id,\[Omega]1},
	idA=GetIdentityTwoCell[A];
	idm=GetIdentityTwoCell[m];
	id=GetIdentityOneCell[GetDest[m]];

	\[Omega]1= Get\[Omega][m,
		OneCellTimes[id,u],
		A
	];

	TwoCellVerticalComposition[TwoCellHorizontalComposition[\[Rho],idA],\[Omega]1]
]

(*IsMonoidAssociative[m,A,\[Mu],\[Alpha],DisplayForm] determines whether \[Mu] satisfies the monoidal/algebra associativity equation as a multiplication on the 1-Cell A.*)
IsMonoidAssociative[m_,A_,\[Mu]_,\[Alpha]_,DisplayForm_:True]:= If[DisplayForm,
	CellByCellTwoMatrixEquation[FullSimplify[Algebra\[Mu]Path1[m,A,\[Mu]]]//TwoToTwoMat,FullSimplify[Algebra\[Mu]Path2[m,A,\[Mu],\[Alpha]]]//TwoToTwoMat]//TwoMatrixForm
	,
	FullSimplify[TwoToTwoMat[Algebra\[Mu]Path1[m,A,\[Mu]]]==TwoToTwoMat[Algebra\[Mu]Path2[m,A,\[Mu],\[Alpha]]]]
]

(*IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho],DisplayForm] determines whether \[Mu] and \[Eta] satisfies the monoidal/algebra right unit equation for \[Eta] to be the unit of the algebra on the 1-Cell A.*)
IsMonoidRightUnit[m_,A_,\[Mu]_,\[Eta]_,u_,\[Rho]_,DisplayForm_:True]:= If[DisplayForm,
	CellByCellTwoMatrixEquation[FullSimplify[Algebra\[Eta]RightPath1[m,A,\[Mu],\[Eta]]]//TwoToTwoMat,FullSimplify[Algebra\[Eta]RightPath2[m,A,\[Mu],u,\[Rho]]]//TwoToTwoMat]//TwoMatrixForm
	,
	FullSimplify[TwoToTwoMat[Algebra\[Eta]RightPath1[m,A,\[Mu],\[Eta]]]==TwoToTwoMat[Algebra\[Eta]RightPath2[m,A,\[Mu],u,\[Rho]]]]
]

(*IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda],DisplayForm] determines whether \[Mu] and \[Eta] satisfies the monoidal/algebra left unit equation for \[Eta] to be the unit of the algebra on the 1-Cell A.*)
IsMonoidLeftUnit[m_,A_,\[Mu]_,\[Eta]_,u_,\[Lambda]_,DisplayForm_:True]:= If[DisplayForm,
	CellByCellTwoMatrixEquation[FullSimplify[Algebra\[Eta]LeftPath1[m,A,\[Mu],\[Eta]]]//TwoToTwoMat,FullSimplify[Algebra\[Eta]LeftPath2[m,A,\[Mu],u,\[Lambda]]]//TwoToTwoMat]//TwoMatrixForm
	,
	FullSimplify[TwoToTwoMat[Algebra\[Eta]LeftPath1[m,A,\[Mu],\[Eta]]]==TwoToTwoMat[Algebra\[Eta]LeftPath2[m,A,\[Mu],u,\[Lambda]]]]
]
	

(*IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] determines whether A, \[Mu], and \[Eta] form a monoid (i.e. an algebra). 
A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell multiplication, and \[Eta] is the 2-Cell unit. 
The structures m, u, \[Alpha], \[Rho], \[Lambda] must form a monoidal category with m the 1-Cell product functor, u the 1-Cell unit, 
\[Alpha] the 2-Cell associator, \[Rho] the 2-Cell right unitor, and \[Lambda] the 2-Cell left unitor.*)
IsMonoid[m_,u_,\[Alpha]_,\[Rho]_,\[Lambda]_,A_,\[Mu]_,\[Eta]_]:=Module[{},
	IsMonoidRightUnit[m,A,\[Mu],\[Eta],u,\[Rho],False]&&
	IsMonoidLeftUnit[m,A,\[Mu],\[Eta],u,\[Lambda],False]&&
	IsMonoidAssociative[m,A,\[Mu],\[Alpha],False]
]

(*path 1 of the equation to determine if the monoid given by A, \[Mu] and \[Eta] form a Dagger Frobenius Monoid*)
DaggerFrobeniusPath1[\[Mu]_,\[Eta]_,A_,m_,\[Alpha]_]:=
Module[{\[Epsilon],\[Delta],id,idA, idm,Leg0, \[Tau]1,LegA,LegB,LegC,\[Omega]2,LegD,\[Tau]2,LegE,LegF},
	\[Epsilon]=GetAdjointTwoCell[\[Eta]];
	\[Delta]=GetAdjointTwoCell[\[Mu]];

	id=GetIdentityOneCell[GetDest[m]];
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];

	Leg0=TwoCellHorizontalComposition[idm, TwoCellTimes[idA,\[Delta]]];

	\[Tau]1 = Get\[Tau][id,
		A,
		m,
		OneCellTimes[A,A]
	];

	LegA=TwoCellHorizontalComposition[idm,\[Tau]1 ];

	LegB= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[OneCellTimes[A,A],A]
	];

	LegC=TwoCellHorizontalComposition[TwoCellInverse[\[Alpha]],TwoCellTimes[TwoCellTimes[idA,idA],idA]];

	\[Omega]2= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[A,A],A]
	];

	(LegD=TwoCellInverse[\[Omega]2]);

	\[Tau]2 = Get\[Tau][m,
		OneCellTimes[A,A],
		id,
		A
	];

	LegE=TwoCellHorizontalComposition[idm, TwoCellInverse[\[Tau]2]];

	LegF=TwoCellHorizontalComposition[idm,TwoCellTimes[\[Mu],idA]];

	TwoCellVerticalComposition[LegF,TwoCellVerticalComposition[LegE,TwoCellVerticalComposition[LegD,TwoCellVerticalComposition[LegC,TwoCellVerticalComposition[LegB,TwoCellVerticalComposition[LegA,Leg0]]]]]]
]

(*path 2 of the equation to determine if the monoid given by A, \[Mu] and \[Eta] form a Dagger Frobenius Monoid*)
DaggerFrobeniusPath2[\[Mu]_,\[Eta]_,A_,m_,\[Alpha]_]:=
Module[{\[Epsilon],\[Delta],id,idA, idm,\[Tau]1, Leg0,LegA,LegB,LegC,\[Omega]2,LegD,\[Tau]2,LegE,LegF},
	\[Epsilon]=GetAdjointTwoCell[\[Eta]];
	\[Delta]=GetAdjointTwoCell[\[Mu]];

	id=GetIdentityOneCell[GetDest[m]];
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];

	Leg0=TwoCellHorizontalComposition[idm, TwoCellTimes[\[Delta],idA]];

	\[Tau]1 = Get\[Tau][m,
		OneCellTimes[A,A],
		id,
		A
	];

	LegA=TwoCellHorizontalComposition[idm,\[Tau]1 ];

	LegB= Get\[Omega][m,
		OneCellTimes[m,id],
		OneCellTimes[OneCellTimes[A,A],A]
	];

	LegC=TwoCellHorizontalComposition[\[Alpha],TwoCellTimes[TwoCellTimes[idA,idA],idA]];

	\[Omega]2= Get\[Omega][m,
		OneCellTimes[id,m],
		OneCellTimes[OneCellTimes[A,A],A]
	];

	(LegD=TwoCellInverse[\[Omega]2]);

	\[Tau]2 = Get\[Tau][id,
		A,
		m,
		OneCellTimes[A,A]
	];

	LegE=TwoCellHorizontalComposition[idm, TwoCellInverse[\[Tau]2]];

	LegF=TwoCellHorizontalComposition[idm,TwoCellTimes[idA,\[Mu]]];

	TwoCellVerticalComposition[LegF,TwoCellVerticalComposition[LegE,TwoCellVerticalComposition[LegD,TwoCellVerticalComposition[LegC,TwoCellVerticalComposition[LegB,TwoCellVerticalComposition[LegA,Leg0]]]]]]
]

(*IsMonoidDaggerFrobenius[m,\[Alpha],A,\[Mu],\[Eta]] determines whether the monoid given by A, \[Mu], and \[Eta] is Dagger-Frobenius. For this function to be called, IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] must be True.*)
IsMonoidDaggerFrobenius[m_,\[Alpha]_,A_,\[Mu]_,\[Eta]_,DisplayForm_:False]:= If[DisplayForm,
	CellByCellTwoMatrixEquation[FullSimplify[DaggerFrobeniusPath1[\[Mu],\[Eta],A,m,\[Alpha]]]//TwoToTwoMat,FullSimplify[DaggerFrobeniusPath2[\[Mu],\[Eta],A,m,\[Alpha]]]//TwoToTwoMat]//TwoMatrixForm
	,
	FullSimplify[TwoToTwoMat[DaggerFrobeniusPath1[\[Mu],\[Eta],A,m,\[Alpha]]]==TwoToTwoMat[DaggerFrobeniusPath2[\[Mu],\[Eta],A,m,\[Alpha]]]]
]

(*Path B to determine whether the monoid formed by A and \[Mu] is commutative. Path A is just \[Mu]*)
CommutativePathB[m_,A_,\[Mu]_,\[Beta]_]:=
Module[{idA,idm,Leg1,\[Omega]1,SAA,Leg2,Leg3,\[Sigma]1},

	(idA=GetIdentityTwoCell[A]);
	(idm=GetIdentityTwoCell[m]);

	Leg1=TwoCellHorizontalComposition[\[Beta],TwoCellTimes[idA,idA]];

	SAA=GetOneCellSwap[GetDest[A],GetDest[A]];

	\[Omega]1= Get\[Omega][m,
		SAA,
		OneCellTimes[A,A]
	];

	Leg2=TwoCellInverse[\[Omega]1];

	\[Sigma]1=Get\[Sigma][A,A];

	Leg3=TwoCellHorizontalComposition[idm,\[Sigma]1];

	TwoCellVerticalComposition[\[Mu],TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[Leg2,Leg1]]]
]

(*IsMonoidCommutative[m,A,\[Mu],\[Beta]] determines whether the monoid given by A and \[Mu] is Commutative. 
For this function to be called, IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] must be True (i.e. A, \[Mu], and \[Eta] must already be shown to form a monoid) 
and the underlying category must be a braided monoidal category.*)
IsMonoidCommutative[m_,A_,\[Mu]_,\[Beta]_,DisplayForm_:False]:= If[DisplayForm,
	CellByCellTwoMatrixEquation[FullSimplify[\[Mu]]//TwoToTwoMat,FullSimplify[CommutativePathB[m,A,\[Mu],\[Beta]]]//TwoToTwoMat]//TwoMatrixForm
	,
	FullSimplify[TwoToTwoMat[\[Mu]]==TwoToTwoMat[CommutativePathB[m,A,\[Mu],\[Beta]]]]
]


(*Monoidal Functor Equations*)

(* First way to go from [(FA (.) FB) (.) FC] to [F ( A (x) (B (x) C) )] which is the first coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation1PathA[F_,\[Phi]_,m1_,\[Alpha]1_,m2_,\[Alpha]2_]:= Module[{idF,id1,id2,Leg1,FFF,\[Omega]1,\[Omega]1Inv,\[Tau]1,\[Tau]1Inv,\[Tau]\[Omega]Leg1,Leg2,\[Tau]2,idm2\[Tau]1Inv,idm2\[Tau]2,\[Omega]2,\[Omega]\[Tau]Leg2,Leg3,\[Omega]3,\[Omega]3Inv,\[Omega]Leg3},
	id1=GetIdentityOneCell[GetDest[m1]];
	id2=GetIdentityOneCell[GetDest[m2]];

	(*compute first path through square *)

	(*first leg and reboxing*)
	FFF=OneCellTimes[OneCellTimes[F,F],F];
	Leg1=WhiskerLeft[\[Alpha]2,FFF];

	(*apply \[Omega] inverse*)
	\[Omega]1= Get\[Omega][m2,
		OneCellTimes[id2,m2],
		FFF
	];
	\[Omega]1Inv = TwoCellInverse[\[Omega]1];

	(*apply \[Tau] inverse*)
	\[Tau]1 = Get\[Tau][id2,
		F,
		m2,
		OneCellTimes[F,F]
	];
	\[Tau]1Inv = TwoCellInverse[\[Tau]1];
	idm2\[Tau]1Inv=WhiskerRight[m2,\[Tau]1Inv];
	\[Tau]\[Omega]Leg1=TwoCellVerticalComposition[idm2\[Tau]1Inv,TwoCellVerticalComposition[\[Omega]1Inv,Leg1]];

	(*compute next Leg and rebox*)
	idF=GetIdentityTwoCell[F];
	Leg2=WhiskerRight[m2,TwoCellTimes[idF,\[Phi]]];

	(*apply \[Tau]*)
	\[Tau]2 = Get\[Tau][F,
		id1,
		F,
		m1
	];
	idm2\[Tau]2=WhiskerRight[m2,\[Tau]2];

	(*apply \[Omega]*)
	\[Omega]2= Get\[Omega][m2,
		OneCellTimes[F,F],
		OneCellTimes[id1,m1]
	];
	\[Omega]\[Tau]Leg2=TwoCellVerticalComposition[\[Omega]2,TwoCellVerticalComposition[idm2\[Tau]2,Leg2]];

	(*compute final leg*)
	Leg3=WhiskerLeft[\[Phi],OneCellTimes[id1,m1]];

	(*compute \[Omega] inverse *)
	\[Omega]3= Get\[Omega][F,
		m1,
		OneCellTimes[id1,m1]
	];
	\[Omega]3Inv = TwoCellInverse[\[Omega]3];
	\[Omega]Leg3=TwoCellVerticalComposition[\[Omega]3Inv,Leg3];
	TwoCellVerticalComposition[\[Omega]Leg3,TwoCellVerticalComposition[\[Omega]\[Tau]Leg2,\[Tau]\[Omega]Leg1]]
]

(* Second way to go from [(FA (.) FB) (.) FC] to [F ( A (x) (B (x) C) )] which is the first coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation1PathB[F_,\[Phi]_,m1_,\[Alpha]1_,m2_,\[Alpha]2_]:= Module[{id1,id2,FFF,idF,\[Omega]4,\[Omega]4Inv,\[Tau]3,\[Tau]3Inv,Leg4,Leg5,Leg6,\[Tau]4,idm2\[Tau]3Inv,idm2\[Tau]4,\[Omega]5,\[Omega]\[Tau]Leg4\[Tau]\[Omega],\[Omega]6,\[Omega]6Inv,\[Omega]Leg5},

	id1=GetIdentityOneCell[GetDest[m1]];
	id2=GetIdentityOneCell[GetDest[m2]];
	FFF=OneCellTimes[OneCellTimes[F,F],F];
	idF=GetIdentityTwoCell[F];

	(*compute the fourth leg (counting from the top path through the square, which has the other three, but start by reboxing *)
	\[Omega]4= Get\[Omega][m2,
		OneCellTimes[m2,id2],
		FFF
	];
	\[Omega]4Inv = TwoCellInverse[\[Omega]4];

	\[Tau]3 = Get\[Tau][m2,
		OneCellTimes[F,F],
		id2,
		F
	];
	\[Tau]3Inv = TwoCellInverse[\[Tau]3];
	idm2\[Tau]3Inv=WhiskerRight[m2,\[Tau]3Inv];
	Leg4=WhiskerRight[m2,TwoCellTimes[\[Phi],idF]];
	\[Tau]4 = Get\[Tau][F,
		m1,
		F,
		id1
	];
	idm2\[Tau]4=WhiskerRight[m2,\[Tau]4];

	\[Omega]5= Get\[Omega][m2,
		OneCellTimes[F,F],
		OneCellTimes[m1,id1]
	];
	\[Omega]\[Tau]Leg4\[Tau]\[Omega]=TwoCellVerticalComposition[\[Omega]5,TwoCellVerticalComposition[idm2\[Tau]4,TwoCellVerticalComposition[Leg4,TwoCellVerticalComposition[idm2\[Tau]3Inv,\[Omega]4Inv]]]];

	(*compute leg5 and reboxing*)
	Leg5=WhiskerLeft[\[Phi],OneCellTimes[m1,id1]];

	\[Omega]6= Get\[Omega][F,
		m1,
		OneCellTimes[m1,id1]
	];
	\[Omega]6Inv = TwoCellInverse[\[Omega]6];
	\[Omega]Leg5=TwoCellVerticalComposition[\[Omega]6Inv,Leg5];

	(*compute leg6*)
	Leg6 = WhiskerRight[F,\[Alpha]1];

	TwoCellVerticalComposition[Leg6,TwoCellVerticalComposition[\[Omega]Leg5,\[Omega]\[Tau]Leg4\[Tau]\[Omega]]]
]

(*Tests if the functor F and structure \[Phi] satsify the first structure equation for F to be a monoidal functor whereby 
[(FA (.) FB) (.) FC] goes to [F ( A (x) (B (x) C) )] in two different ways *)
IsMonoidalFunctorStructureEquation1[F_,\[Phi]_,m1_,\[Alpha]1_,m2_,\[Alpha]2_,WithErrors_:True,DisplayForm_:True]:=Module[{\[Phi]Source,\[Phi]Dest},
	If[WithErrors,	
		\[Phi]Source=Generate\[Phi]Source[F,m2];
		\[Phi]Dest=Generate\[Phi]Dest[F,m1];

		If[\[Phi]Source!=\[Phi]Dest,
			Print["Error: F can never be a monoidal functor for the monoidal categories given by (x), m1, and (.), m2. [FA (.) FB] does not give the same functor as [F(A (x) B)]."];
			False,

			If[\[Phi]Source!=GetSource[\[Phi]],
				Print["Error: m2 and F, and \[Phi] are incompatible. The functor given by [FA (.) FB] is not the same as source OneCell of \[Phi]."];
				False,

				If[\[Phi]Dest!=GetDest[\[Phi]],
					Print["Error: m1 and F, and \[Phi] are incompatible. The functor given by [F(A (x) B)] is not the same as destination OneCell of \[Phi]."];
					False,

					If[!IsTwoCellInvertible[\[Phi]],
						Print["Error: \[Phi] is not invertible and thus is not an isomorphism. Thus, F cannot be a strong monoidal functor."];
						False,

						If[FullSimplify[MonoidalFunctorStructureEquation1PathA[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]==MonoidalFunctorStructureEquation1PathB[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]],
							True,
							Print["All the structures are compatible but do not satisfy the first monoidal functor structure equation."];
							False
						]
					]
				]
			]
		]
		,
		If[DisplayForm,
			CellByCellTwoMatrixEquation[FullSimplify[MonoidalFunctorStructureEquation1PathA[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]]//TwoToTwoMat,FullSimplify[MonoidalFunctorStructureEquation1PathB[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]]//TwoToTwoMat]//TwoMatrixForm
			,
			FullSimplify[TwoToTwoMat[MonoidalFunctorStructureEquation1PathA[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]]==TwoToTwoMat[MonoidalFunctorStructureEquation1PathB[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2]]]
		]
	]
]

(* Long way to go from [FA (.) u2] to [FA], which is the second coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation2PathA[F_,\[Phi]_,\[Phi]u_,m1_,\[Rho]1_,u1_,m2_,u2_]:=Module[{id1,id2,Leg1,idF,\[Tau]1,idm2\[Tau]1,\[Omega]1,\[Omega]\[Tau]Leg1,Leg2,\[Omega]2,\[Omega]2Inv,\[Omega]Leg2,Leg3},
	id1=GetIdentityOneCell[GetDest[m1]];
	id2=GetIdentityOneCell[GetDest[m2]];

	(*compute long side first*)

	(* leg1 *)
	idF=GetIdentityTwoCell[F];
	Leg1=WhiskerRight[m2,TwoCellTimes[idF,\[Phi]u]];

	(*apply \[Tau]*)
	\[Tau]1 = Get\[Tau][F,
		id2,
		F,
		u1
	];
	idm2\[Tau]1=WhiskerRight[m2, \[Tau]1];

	(*apply \[Omega]*)
	\[Omega]1= Get\[Omega][m2,
		OneCellTimes[F,F],
		OneCellTimes[id1,u1]
	];
	\[Omega]\[Tau]Leg1=TwoCellVerticalComposition[\[Omega]1,TwoCellVerticalComposition[idm2\[Tau]1,Leg1]];

	(* leg2 *)
	Leg2=WhiskerLeft[\[Phi],OneCellTimes[id1,u1]];

	(*apply \[Omega] inverse*)
	\[Omega]2= Get\[Omega][F,
		m1,
		OneCellTimes[id1,u1]
	];
	\[Omega]2Inv=TwoCellInverse[\[Omega]2];
	\[Omega]Leg2=TwoCellVerticalComposition[\[Omega]2Inv,Leg2];

	(*compute leg3*)
	Leg3=WhiskerRight[F,\[Rho]1];

	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[\[Omega]Leg2,\[Omega]\[Tau]Leg1]]
]

(* Short way to go from [FA (.) u2] to [FA], which is the second coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation2PathB[F_,m2_,\[Rho]2_,u2_]:=Module[{id2,\[Omega]3,Leg4},
	id2=GetIdentityOneCell[GetDest[m2]];
	\[Omega]3= Get\[Omega][m2,
		OneCellTimes[id2,u2],
		F
	];
	Leg4=WhiskerLeft[\[Rho]2,F];

	TwoCellVerticalComposition[Leg4,\[Omega]3]
]

(* Long way to go from [u2 (.) FB] to [FB], which is the third coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation3PathA[F_,\[Phi]_,\[Phi]u_,m1_,\[Lambda]1_,u1_,m2_,u2_]:=Module[{id1,id2,Leg1,idF,\[Tau]1,idm2\[Tau]1,\[Omega]1,\[Omega]\[Tau]Leg1,Leg2,\[Omega]2,\[Omega]2Inv,\[Omega]Leg2,Leg3,},

	id1=GetIdentityOneCell[GetDest[m1]];
	id2=GetIdentityOneCell[GetDest[m2]];

	(* leg1 *)
	idF=GetIdentityTwoCell[F];
	Leg1=WhiskerRight[m2,TwoCellTimes[\[Phi]u,idF]];

	(*apply \[Tau]*)
	\[Tau]1 = Get\[Tau][F,
		u1,
		F,
		id2
	];
	idm2\[Tau]1=WhiskerRight[m2, \[Tau]1];

	(*apply \[Omega]*)
	\[Omega]1= Get\[Omega][m2,
		OneCellTimes[F,F],
		OneCellTimes[u1,id1]
	];
	\[Omega]\[Tau]Leg1=TwoCellVerticalComposition[\[Omega]1,TwoCellVerticalComposition[idm2\[Tau]1,Leg1]];

	(* leg2 *)
	Leg2=WhiskerLeft[\[Phi],OneCellTimes[u1,id1]];

	(*apply \[Omega] inverse*)
	\[Omega]2= Get\[Omega][F,
		m1,
		OneCellTimes[u1,id1]
	];
	\[Omega]2Inv=TwoCellInverse[\[Omega]2];
	\[Omega]Leg2=TwoCellVerticalComposition[\[Omega]2Inv,Leg2];

	(*compute leg3*)
	Leg3=WhiskerRight[F,\[Lambda]1];

	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[\[Omega]Leg2,\[Omega]\[Tau]Leg1]]
]

(* Short way to go from [u2 (.) FB] to [FB], which is the third coherence map or structure equation that a monoidal functor, F, needs to satisfy *)
MonoidalFunctorStructureEquation3PathB[F_,m2_,\[Lambda]2_,u2_]:=Module[{id2,\[Omega]3,Leg4},
	id2=GetIdentityOneCell[GetDest[m2]];
	\[Omega]3= Get\[Omega][m2,
		OneCellTimes[u2,id2],
		F
	];
	Leg4=WhiskerLeft[\[Lambda]2,F];

	TwoCellVerticalComposition[Leg4,\[Omega]3]
]

(*Tests if the functor F and structures \[Phi]u and \[Phi] satsify the second and third structure equations for F to be 
a strong monoidal functor whereby [FA (.) u2] goes to [FA] in two different ways and [u2 (.) FB] goes to [FB] in two different ways. *)
(*Note: the first monoidal functor structure equation involving F and \[Phi] must be shown to be satisfied first before using this one*)
(*Note: some of the errors are sequential, and thus all the possible errors might not be shown. For instance, if both the second and 
third monoidal structure equations are not satisfied, it will only inform the user about the first not being satisfied*)
IsMonoidalFunctorStructureEquation2and3[F_,\[Phi]_,\[Phi]u_,m1_,\[Alpha]1_,\[Rho]1_,\[Lambda]1_,u1_,m2_,\[Alpha]2_,\[Rho]2_,\[Lambda]2_,u2_,WithErrors_:True,DisplayForm_:True]:=Module[{\[Phi]uSource,\[Phi]uDest},
	If[WithErrors,	
		\[Phi]uSource=Generate\[Phi]uSource[u2];
		\[Phi]uDest=Generate\[Phi]uDest[F,u1];

		If[\[Phi]uSource!=\[Phi]uDest,
			Print["Error: F can never be a monoidal functor for the monoidal categories given by (x), m1, and (.), m2. The OneCell u2 is not the same as one OneCell F(u1)."];
			False,

			If[\[Phi]uSource!=GetSource[\[Phi]u],
				Print["Error: u2 and \[Phi]u are incompatible. The OneCell given by us is not the same as source OneCell of \[Phi]u."];
				False,

				If[\[Phi]uDest!=GetDest[\[Phi]u],
					Print["Error: u1 and F, and \[Phi]u are incompatible. The OneCell given by F(u1) is not the same as destination OneCell of \[Phi]u."];
					False,

					If[!IsTwoCellInvertible[\[Phi]u],
						Print["Error: \[Phi]u is not invertible and thus is not an isomorphism. Thus, F cannot be a strong monoidal functor."];
						False,

						If[FullSimplify[MonoidalFunctorStructureEquation2PathA[F,\[Phi],\[Phi]u,m1,\[Rho]1,u1,m2,u2]!=MonoidalFunctorStructureEquation2PathB[F,m2,\[Rho]2,u2]],
							Print["All the structures are compatible but do not satisfy the second monoidal functor structure equation."];
							False,
							If[FullSimplify[MonoidalFunctorStructureEquation3PathA[F,\[Phi],\[Phi]u,m1,\[Lambda]1,u1,m2,u2]!=MonoidalFunctorStructureEquation3PathB[F,m2,\[Lambda]2,u2]],
								Print["All the structures are compatible but do not satisfy the third monoidal functor structure equation."];
								False,
								True
							]
						]
					]
				]
			]
		]
		,
		If[DisplayForm,
			Print["Equation 2: ", CellByCellTwoMatrixEquation[FullSimplify[MonoidalFunctorStructureEquation2PathA[F,\[Phi],\[Phi]u,m1,\[Rho]1,u1,m2,u2]]//TwoToTwoMat,FullSimplify[MonoidalFunctorStructureEquation2PathB[F,m2,\[Rho]2,u2]]//TwoToTwoMat]//TwoMatrixForm];
			Print["Equation 3: ", CellByCellTwoMatrixEquation[FullSimplify[MonoidalFunctorStructureEquation3PathA[F,\[Phi],\[Phi]u,m1,\[Lambda]1,u1,m2,u2]]//TwoToTwoMat,FullSimplify[MonoidalFunctorStructureEquation3PathB[F,m2,\[Lambda]2,u2]]//TwoToTwoMat]//TwoMatrixForm];
			,
			Flatten[{
				CellByCellTwoMatrixEquation[FullSimplify[MonoidalFunctorStructureEquation2PathA[F,\[Phi],\[Phi]u,m1,\[Rho]1,u1,m2,u2]]//TwoToTwoMat,FullSimplify[MonoidalFunctorStructureEquation2PathB[F,m2,\[Rho]2,u2]]//TwoToTwoMat]
				,
				CellByCellTwoMatrixEquation[FullSimplify[MonoidalFunctorStructureEquation3PathA[F,\[Phi],\[Phi]u,m1,\[Lambda]1,u1,m2,u2]]//TwoToTwoMat,FullSimplify[MonoidalFunctorStructureEquation3PathB[F,m2,\[Lambda]2,u2]]//TwoToTwoMat]
			}]
		]
	]
]

(* Tests if proposed structures form a monoidal functor, F, from the category given by product functor m1 to the category given by product functor m2 
(where structures labeled by 1 are from the category given by m1 and structures labaled by 2 are from the category given by m2. *)
(* Note: There is an order to which the errors happen instead of giving all of the possible ones that apply. 
But this makes sense, as the first monoidal structure equation needs to be shown to be satisfied first before the 
function that tests the second and thrid structure equations is called. *)
IsStrongMonoidalFunctor[F_,\[Phi]_,\[Phi]u_,m1_,\[Alpha]1_,\[Rho]1_,\[Lambda]1_,u1_,m2_,\[Alpha]2_,\[Rho]2_,\[Lambda]2_,u2_]:=Module[{},

	If[!IsMonoidalFunctorStructureEquation1[F,\[Phi],m1,\[Alpha]1,m2,\[Alpha]2],
		Print["First monoidal functor structure equation is not satisfied."];
		False,

		If[!IsMonoidalFunctorStructureEquation2and3[F,\[Phi],\[Phi]u,m1,\[Alpha]1,\[Rho]1,\[Lambda]1,u1,m2,\[Alpha]2,\[Rho]2,\[Lambda]2,u2],
			Print["First monoidal functor structure equation is satisfied but the second or third monoidal functor structure equations or both (the ones that deal with \[Phi]u) are not."];
			False,
			True
		]
	]
]



(*Solvers (prototypes) *)

SolveMonoidal[m_,u_]:=Module[{\[Alpha],\[Rho],\[Lambda],a,b,c,d,e,f,g},
	\[Alpha]=GenerateVariable\[Alpha][m];
	\[Rho]=GenerateVariable\[Rho][m,u];
	\[Lambda]=GenerateVariable\[Lambda][m,u];

	Print["\[Alpha] = ",\[Alpha]//TwoCellForm];
	Print["\[Rho] = ",\[Rho]//TwoCellForm];
	Print["\[Lambda] = ",\[Lambda]//TwoCellForm];

	a=Rebracket1[m,\[Alpha]]//TwoToTwoMat;
	b=Rebracket2[m,\[Alpha]]//TwoToTwoMat;
	c=IsTwoCellInvertible[\[Alpha]];
	d=TrianglePathA[m,\[Rho]]//TwoToTwoMat;
	e=TrianglePathB[m,\[Alpha],\[Lambda],u]//TwoToTwoMat;
	f=IsTwoCellInvertible[\[Rho]];
	g=IsTwoCellInvertible[\[Lambda]];

	Solve[(a==b)&&(d==e)&&c&&f&&g]
]


(* Helper function to sum the simple objects in a 1-cell with source 1 *)
SumSimples[l_]:=Total[l[[1]]][[1]]

(*Automatic reboxing functions for when the 2-Cell is to be applied to specific objects*)

(*\[Alpha]ABC[m,\[Alpha],A,B,C] returns the 2-Cell given by \[Alpha]_{A,B,C} (i.e. \[Alpha] applied to objects A, B, and C), 
where A, B, and C are 1-Cell objects of the monoidal category, and m is the 1-Cell product functor.
 The output 2-Cell is automatically appropriately reboxed.*)
\[Alpha]ABC[m_, \[Alpha]_, A_, B_, C_] := 
 Module[{id, idm, idA, idB, 
   idC, \[Tau]1, \[Tau]2, \[Omega]1, \[Omega]2, Leg1, Leg2, Leg3, 
   Leg4, Leg5}, idm = GetIdentityTwoCell[m];
  idA = GetIdentityTwoCell[A];
  idB = GetIdentityTwoCell[B];
  idC = GetIdentityTwoCell[C];
  id = GetIdentityOneCell[GetDest[m]];
  (*Decide if we're feeding this simple objects*)
  Leg3 = TwoCellHorizontalComposition[\[Alpha], 
    TwoCellTimes[idA, TwoCellTimes[idB, idC]]]; 
  If[SumSimples[A]*SumSimples[B]*SumSimples[C] == 1,
   Leg3,
   (*\[Tau]1=Get\[Tau][m,OneCellTimes[A,B],id,C];
   Leg1=TwoCellHorizontalComposition[
   idm,\[Tau]1];*)\[Omega]1 = 
    Get\[Omega][m, OneCellTimes[m, id], 
     OneCellTimes[OneCellTimes[A, B], C]];
   Leg2 = \[Omega]1;
   \[Omega]2 = 
    Get\[Omega][m, OneCellTimes[id, m], 
     OneCellTimes[OneCellTimes[A, B], C]];
   Leg4 = TwoCellInverse[\[Omega]2];
   (*\[Tau]2=Get\[Tau][id,A,m,OneCellTimes[B,C]];
   Leg5=TwoCellHorizontalComposition[idm,
   TwoCellInverse[\[Tau]2]];*)(*TwoCellVerticalComposition[Leg5,
   TwoCellVerticalComposition[Leg4,TwoCellVerticalComposition[Leg3,
   TwoCellVerticalComposition[Leg2,Leg1]]]]*)
   Leg4\[CenterDot]Leg3\[CenterDot]Leg2
   ]
  ]
  
(*\[Beta]AB[m,\[Beta],A,B] returns the 2-Cell given by \[Beta]_{A,B} (i.e. \[Beta] applied to objects A and B), 
where A and B are 1-Cell objects of the braided monoidal category, and m is the 1-Cell product functor. 
The output 2-Cell is automatically appropriately reboxed.*)
\[Beta]AB[m_,\[Beta]_,A_,B_]:=
Module[{id,idm,idA,idB,Leg1,\[Omega]1,Leg2,Leg3},
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];
	idB=GetIdentityTwoCell[B];

	Leg1 = TwoCellHorizontalComposition[\[Beta],TwoCellTimes[idA,idB]];

	\[Omega]1= Get\[Omega][m,
		GetOneCellSwap[GetDest[A],GetDest[B]],
		OneCellTimes[A,B]
	];

	Leg2=TwoCellInverse[\[Omega]1];

	Leg3=TwoCellHorizontalComposition[idm,Get\[Sigma][A,B]];

	TwoCellVerticalComposition[Leg3,TwoCellVerticalComposition[Leg2,Leg1]]
]

(*\[Theta]A[m,\[Theta],A] returns the 2-Cell given by \[Theta]_A (i.e. \[Theta] applied to object A), 
where A is a 1-Cell object of the twisted braided monoidal category, and m is the 1-Cell product functor. 
The output 2-Cell is automatically appropriately reboxed.*)
\[Theta]A[m_,\[Theta]_,A_]:=
Module[{idA},
	idA=GetIdentityTwoCell[A];
	TwoCellHorizontalComposition[\[Theta],idA]
]

(*\[Lambda]A[m,\[Lambda],u,A] returns the 2-Cell given by \[Lambda]_A (i.e. \[Lambda] applied to object A),
 where A is a 1-Cell object of the monoidal category, m is the 1-Cell product functor, 
and u is the 1-Cell unit object. The output 2-Cell is automatically appropriately reboxed.*)
\[Lambda]A[m_,\[Lambda]_,u_,A_]:=
Module[{id,idm,idA,\[Omega]1,Leg1,Leg2},
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];
	id=GetIdentityOneCell[GetDest[m]];

	\[Omega]1= Get\[Omega][m,
		OneCellTimes[u,id],
		A
	];

	Leg1=\[Omega]1;

	Leg2=TwoCellHorizontalComposition[\[Lambda], idA];

	TwoCellVerticalComposition[Leg2,Leg1]
]

(*\[Rho]A[m,\[Rho],u,A] returns the 2-Cell given by \[Rho]_A (i.e. \[Rho] applied to object A),
 where A is a 1-Cell object of the monoidal category, m is the 1-Cell product functor, 
and u is the 1-Cell unit object. The output 2-Cell is automatically appropriately reboxed.*)
\[Rho]A[m_,\[Rho]_,u_,A_]:=
Module[{id,idm,idA,\[Omega]1,Leg1,Leg2},
	idm=GetIdentityTwoCell[m];
	idA=GetIdentityTwoCell[A];
	id=GetIdentityOneCell[GetDest[m]];

	\[Omega]1= Get\[Omega][m,
		OneCellTimes[id,u],
		A
	];

	Leg1=\[Omega]1;

	Leg2=TwoCellHorizontalComposition[\[Rho], idA];

	TwoCellVerticalComposition[Leg2,Leg1]
]

(*\[Phi]AB[m1,m2,F,\[Phi],A,B] returns the 2-Cell given by \[Phi]_{A,B} (i.e. \[Phi] applied to objects A and B),
 where A and B are 1-Cell objects of the monoidal category given by m1. F is the 1-Cell monoidal functor,
 m1 is the 1-Cell product functor of the first category, and m2 is the 1-Cell product functor of the second category.
 The output 2-Cell is automatically appropriately reboxed.*)
\[Phi]AB[m1_,m2_,F_,\[Phi]_,A_,B_]:=
Module[{idm2,idA,idB,\[Tau]1,Leg1,\[Omega]1,Leg2,Leg3,Leg4,\[Omega]2},

	idm2=GetIdentityTwoCell[m2];
	idA=GetIdentityTwoCell[A];
	idB=GetIdentityTwoCell[B];


	\[Tau]1 = Get\[Tau][F,
		A,
		F,
		B
	];

	Leg1= TwoCellHorizontalComposition[idm2,\[Tau]1];

	\[Omega]1= Get\[Omega][m2,
		OneCellTimes[F,F],
		OneCellTimes[A,B]
	];

	Leg2= \[Omega]1;

	Leg3=TwoCellHorizontalComposition[\[Phi],TwoCellTimes[idA,idB]];

	\[Omega]2= Get\[Omega][F,
		m1,
		OneCellTimes[A,B]
	];

	Leg4=TwoCellInverse[\[Omega]2];

	TwoCellVerticalComposition[Leg4, TwoCellVerticalComposition[Leg3, TwoCellVerticalComposition[Leg2,Leg1]]]
]



End[ ]

EndPackage[ ]




