(* ::Package:: *)

(*

A Mathematica Package for 2Vect and Modular Tensor Categories

Daniel A. Roberts
Department of Computer Science
University of Oxford

MTCategory.m

*)
BeginPackage["MTCategory`", {"TwoVect`", "MTCategories`"}]

SetMTC::usage =
		"SetMTC[m,u] is used to set the modular tensor category you want to work with. The 2-Cell structures are created in the appropriate form with variable entries. Please input m, the 1-Cell fusion rules, and u, the 1-Cell unit. SetMTC[m,u,\[Alpha]Set,\[Rho]Set,\[Lambda]Set,\[Beta]Set,\[Theta]Set] is used to set the modular tensor category with user inputted 2-Cell structures. Please run IsMTC[] to ensure that the structures satisfy all the MTC axioms. As before, m, the 1-Cell fusion rules, and u, the 1-Cell unit, must be inputted as 1-Cells."

SetMTCFib::usage =
		"SetMTCFib[] is used to set the modular tensor category you want to work with to Fib, the Fibonacci MTC."

SetMTCIsing::usage =
		"SetMTCIsing[] is used to set the modular tensor category you want to work with to the Ising MTC."

Set\[Alpha]::usage =
		"Set\[Alpha][\[Alpha]Set] sets the 2-Cell \[Alpha], the associator of the MTC. Input must be the appropriate 2-Cell. Please run IsMTC[] to ensure that the new structure satisfies all the MTC axioms."

Set\[Rho]::usage =
		"Set\[Rho][\[Rho]Set] sets the 2-Cell \[Rho], the right unitor of the MTC. Input must be the appropriate 2-Cell. Please run IsMTC[] to ensure that the new structure satisfies all the MTC axioms."

Set\[Lambda]::usage =
		"Set\[Lambda][\[Lambda]set] sets the 2-Cell \[Lambda], the left unitor of the MTC. Input must be the appropriate 2-Cell. Please run IsMTC[] to ensure that the new structure satisfies all the MTC axioms."

Set\[Beta]::usage =
		"Set\[Beta][\[Beta]Set] sets the 2-Cell \[Beta], the braid of the MTC. Input must be the appropriate 2-Cell. Please run IsMTC[] to ensure that the new structure satisfies all the MTC axioms."

Set\[Theta]::usage =
		"Set\[Theta][\[Theta]Set] sets the 2-Cell \[Theta], the twist of the MTC. Input must be the appropriate 2-Cell. Please run IsMTC[] to ensure that the new structure satisfies all the MTC axioms."

PrintMTC::usage =
		"PrintMTC[] is used to print all the structures of the current MTC. It is not checked for consistency or form. Please run IsMTC[] to ensure correct form and to ensure that the structures satisfy all the MTC axioms."

IsMTC::usage =
		"IsMTC[] tests if the currently set MTC satisfies all the axioms. IsMTC[m,\[Alpha],\[Rho],\[Lambda],u,\[Beta],\[Theta]] tests if the input structures form a modular tensor category, where  where m is the 1-Cell product functor, \[Alpha] is the 2-Cell associator, \[Rho] is the 2-Cell right unitor, \[Lambda] is the 2-Cell left unitor, u is the 1-Cell unit, \[Beta] is the 2-Cell braid, and \[Theta] is the 2-Cell twist."

GenerateHandle::usage =
		"GenerateHandle[] generates the handle (which is a 1-Cell object) of the currently set MTC. GenerateHandle[m,u] generates the handle of the MTC given by product functor m and unit u. The handle is the sum over all the simple objects of that simple object fused with its dual: \[CapitalSigma] A (x) A*, for all simple objects A"

IsMonoid::usage =
		"IsMonoid[A,\[Mu],\[Eta]] tests if A, \[Mu], and \[Eta] form a monoid (i.e. an algebra), where A is a 1-Cell the algebra is over (and is an object of the currently set MTC), \[Mu] is the 2-Cell multiplication, and \[Eta] is the 2-Cell monoid unit. IsMonoid[m,u,\[Alpha],\[Rho],\[Lambda],A,\[Mu],\[Eta]] tests whether A, \[Mu], and \[Eta] form a monoid in the monoidal category given by m, u, \[Alpha], \[Rho], and \[Lambda] (where m is the 1-Cell product functor, u is the 1-Cell unit, \[Alpha] is the 2-Cell associator, \[Rho] is the 2-Cell right unitor, and \[Lambda] is the 2-Cell left unitor). Note: does not specify the error (i.e. whether there is a problem with the form of A, \[Mu], or \[Eta], or whether the equation fails) if the function returns False."

IsMonoidDaggerFrobenius::usage =
		"IsMonoidDaggerFrobenius[A,\[Mu],\[Eta]] determines whether the monoid given by A, \[Mu], and \[Eta] is Dagger-Frobenius, where A is a 1-Cell in the current category (and is the object the monoid is over), \[Mu] is the 2-Cell multiplication, and \[Eta] is the 2-Cell monoid unit. Note: for this function to be called, A, \[Mu], and \[Eta] must already be shown to form a monoid."

IsMonoidCommutative::usage = 
		"IsMonoidCommutative[A,\[Mu]] determines whether the monoid given by A and \[Mu] (\[Eta] is unneeded for this function) is Commutative, where A a 1-Cell object in the current category (and is the object the monoid is over), and \[Mu] is the 2-Cell multiplication. Note: for this function to be called, A, \[Mu], and \[Eta] must already be shown to form a monoid."

Getm::usage =
		"Getm[] returns the 1-Cell m, the fusion rules of the MTC."

Getu::usage =
		"Getu[] returns the 1-Cell u, the unit of the MTC."

Get\[Alpha]::usage =
		"Get\[Alpha][] returns the 2-Cell \[Alpha], the associator of the MTC."

Get\[Rho]::usage =
		"Get\[Rho][] returns the 2-Cell \[Rho], the right unitor of the MTC."

Get\[Lambda]::usage =
		"Get\[Lambda][] returns the 2-Cell \[Lambda], the left unitor of the MTC."

Get\[Beta]::usage =
		"Get\[Beta][] returns the 2-Cell \[Beta], the braid of the MTC."

Get\[Theta]::usage =
		"Get\[Theta][] returns the 2-Cell \[Theta], the twist of the MTC."

GetPentagonEquation::usage =
		"GetPentagonEquation[DisplayForm] returns the pentagon equation, an axiom of monoidal categories (and thus MTCs). GetPentagonEquation[] or GetPentagonEquation[True] prints the axiom in display form. GetPentagonEquation[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True. GetPentagonEquation[DisplayForm] is essentially intended to be used when \[Alpha] has variable entries."

GetTriangleEquation::usage =
		"GetTriangleEquation[DisplayForm] returns the triangle equation, an axiom of monoidal categories (and thus MTCs). GetTriangleEquation[] or GetTriangleEquation[True] prints the axiom in display form. GetTriangleEquation[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True. GetTriangleEquation[DisplayForm] is essentially intended to be used when \[Alpha], \[Lambda], and/or \[Rho] have variable entries."

GetHexagonEquation1::usage =
		"GetHexagonEquation1[DisplayForm] returns the first hexagon equation for braided monoidal categories, an axiom of MTCs. GetHexagonEquation1[] or GetHexagonEquation1[True] prints the axiom in display form. GetHexagonEquation1[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True. GetHexagonEquation1[DisplayForm] is essentially intended to be used when \[Beta] and/or \[Alpha] have variable entries."

GetHexagonEquation2::usage =
		"GetHexagonEquation2[DisplayForm] returns the first hexagon equation for braided monoidal categories, an axiom of MTCs. GetHexagonEquation2[] or GetHexagonEquation2[True] prints the axiom in display form. GetHexagonEquation2[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True. GetHexagonEquation2[DisplayForm] is essentially intended to be used when \[Beta] has variable entries, but MUST NOT be used when \[Alpha] has variable entries since \[Alpha] inverse is required."

GetTwistEquation::usage =
		"GetTwistEquation[DisplayForm] returns the twist equation, an axiom of MTCs. GetTwistEquation[] or GetTwistEquation[True] prints the axiom in display form. GetTwistEquation[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True. GetTwistEquation[DisplayForm] is essentially intended to be used when \[Beta] and/or \[Theta] have variable entries."

GetRibbonCoherenceEquation::usage =
		"GetRibbonCoherenceEquation[DisplayForm] returns the equation of coherence between the 2-Cell twist \[Theta] and the dual structure of the MTC. This is a required axiom of MTCs. GetRibbonCoherenceEquation[] or GetRibbonCoherenceEquation[True] prints the axiom on display form. GetRibbonCoherenceEquation[False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

GetMonoidAssociativityEquation::usage =
		"GetMonoidAssociativityEquation[A,\[Mu],DisplayForm] returns the associativity equation for A and \[Mu] to form a monoid, where A is the 1-Cell object the algebra is over, and \[Mu] is the 2-Cell monoid multiplication. GetMonoidAssociativityEquation[A,\[Mu]] or GetMonoidAssociativityEquation[A,\[Mu],True] prints the axiom on display form. GetMonoidAssociativityEquation[A,\[Mu],False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

GetMonoidRightUnitEquation::usage =
		"GetMonoidRightUnitEquation[A,\[Mu],\[Eta],DisplayForm] returns the right unit equation for A, \[Mu], and \[Eta] to form a monoid, where A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell monoid multiplication, and \[Eta] is the 2-Cell monoid unit. GetMonoidRightUnitEquation[A,\[Mu],\[Eta]] or GetMonoidRightUnitEquation[A,\[Mu],\[Eta],True] prints the axiom on display form. GetMonoidRightUnitEquation[A,\[Mu],\[Eta],False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

GetMonoidLeftUnitEquation::usage =
		"GetMonoidLeftUnitEquation[A,\[Mu],\[Eta],DisplayForm] returns the left unit equation for A, \[Mu], and \[Eta] to form a monoid, where A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell monoid multiplication, and \[Eta] is the 2-Cell monoid unit. GetMonoidLeftUnitEquation[A,\[Mu],\[Eta]] or GetMonoidLeftUnitEquation[A,\[Mu],\[Eta],True] prints the axiom on display form. GetMonoidLeftUnitEquation[A,\[Mu],\[Eta],False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

GetMonoidDaggerFrobeniusEquation::usage =
		"GetMonoidDaggerFrobeniusEquation[A,\[Mu],\[Eta],DisplayForm] returns the dagger-Frobenius equation, the conditions required for the monoid formed by  A, \[Mu], and \[Eta] to be dagger-Frobenius (where A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell monoid multiplication, and \[Eta] is the 2-Cell monoid unit). GetMonoidDaggerFrobeniusEquation[A,\[Mu],\[Eta]] or GetMonoidDaggerFrobeniusEquation[A,\[Mu],\[Eta],True] prints the axiom on display form. GetMonoidDaggerFrobeniusEquation[A,\[Mu],\[Eta],False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

GetMonoidCommutativityEquation::usage =
		"GetMonoidCommutativityEquation[A,\[Mu],DisplayForm] returns the commutativity equation, the conditions required for the monoid formed by A, \[Mu], and \[Eta] to be commutative (where A is the 1-Cell object the algebra is over, \[Mu] is the 2-Cell monoid multiplication, and \[Eta] is the 2-Cell monoid unit). GetMonoidCommutativityEquation[A,\[Mu]] or GetMonoidCommutativityEquation[A,\[Mu],True] prints the axiom on display form. GetMonoidCommutativityEquation[A,\[Mu],False] returns the equation in input form, which can then be given to a solver. DisplayForm is an optional argument that defaults to True."

\[Alpha]::usage =
		"\[Alpha][A,B,C] returns the 2-Cell associator given by \[Alpha]_{A,B,C} (i.e. \[Alpha] applied to objects A, B, and C), where A, B, and C are 1-Cell objects of the monoidal category. The output 2-Cell is automatically appropriately reboxed."

\[Beta]::usage =
		"\[Beta][A,B] returns the 2-Cell braid given by \[Beta]_{A,B} (i.e. \[Beta] applied to objects A and B), where A and B are 1-Cell objects of the braided monoidal category. The output 2-Cell is automatically appropriately reboxed."

\[Theta]::usage =
		"\[Theta][A] returns the 2-Cell twist given by \[Theta]_A (i.e. \[Theta] applied to object A), where A is a 1-Cell object of the twisted braided monoidal category. The output 2-Cell is automatically appropriately reboxed."

\[Lambda]::usage =
		"\[Lambda][A] returns the 2-Cell left unitor given by \[Lambda]_A (i.e. \[Lambda] applied to object A), where A is a 1-Cell object of the monoidal category. The output 2-Cell is automatically appropriately reboxed."

\[Rho]::usage =
		"\[Rho][A] returns the 2-Cell right unitor given by \[Rho]_A (i.e. \[Rho] applied to object A), where A is a 1-Cell object of the monoidal category. The output 2-Cell is automatically appropriately reboxed."

CircleTimes::usage =
		"A\[CircleTimes]B or CircleTimes[A,B] monoidally combines in the underlying MTC. If A and B are 1-Cells (i.e. objects in the underlying MTC), then this is given by OneCellComposition[m, OneCellTimes[A,B]]. If A and B are 2-Cells (i.e. morphisms in the underlying MTC), then this is given by TwoCellHorizontalComposition[id[m], TwoCellTimes[A,B]]. For both, m represents the fusion rules of the underlying category. If the arguments are ALL 1-Cells, this is an n-ary operation A\[CircleTimes]B\[CircleTimes]..., if the arguments are 2-Cells the operation is only binary (so please use appropriate bracketing)."



Begin["`Private`"]


(*Private functions from the MTCategory project, repeated so as to not make them public*)

(* Return the source of \[Alpha], the associator, from input of m, the monodial functor *)
Generate\[Alpha]Source[m_]:=OneCellComposition[m,OneCellTimes[m,GetIdentityOneCell[GetDest[m]]]]

(* Return the destination of \[Alpha], the associator, from input of m, the monodial functor *)
Generate\[Alpha]Dest[m_]:=OneCellComposition[m,OneCellTimes[GetIdentityOneCell[GetDest[m] ],m] ]

(* return source of \[Rho] from m and u, the unit *)
Generate\[Rho]Source[m_,u_]:=OneCellComposition[m,OneCellTimes[GetIdentityOneCell[GetDest[m]],u]]

(* return Destination, which is the identity OneCell *)
Generate\[Rho]Dest[m_,u_]:=GetIdentityOneCell[GetDest[m]]

(* return source of \[Rho] from m and u, the unit *)
Generate\[Lambda]Source[m_,u_]:=OneCellComposition[m,OneCellTimes[u,GetIdentityOneCell[GetDest[m]]]]

(* return domain, which is the identity OneCell *)
Generate\[Lambda]Dest[m_,u_]:=GetIdentityOneCell[GetDest[m]]

(* Returns a completely variable \[Alpha] (with the correct OneCell source and destination) for a given m *)
GenerateVariable\[Alpha][m_]:=GenerateVariableTwoCell[Generate\[Alpha]Source[m],Generate\[Alpha]Dest[m]]

(* Returns a completely variable \[Rho] (with the correct OneCell source and destination) for a given m and u *)
GenerateVariable\[Rho][m_,u_]:=GenerateVariableTwoCell[Generate\[Rho]Source[m,u],Generate\[Rho]Dest[m,u]]

(* Returns a completely variable \[Lambda] (with the correct OneCell source and destination) for a given m and u*)
GenerateVariable\[Lambda][m_,u_]:=GenerateVariableTwoCell[Generate\[Lambda]Source[m,u],Generate\[Lambda]Dest[m,u]]

(* Returns the source of \[Beta], the braid transformation, from input of m, the monodial functor *)
Generate\[Beta]Source[m_]:=m

(* Returns the destination of \[Beta], the braid transformation, from input of m, the monodial functor *)
Generate\[Beta]Dest[m_]:=OneCellComposition[m,GetOneCellSwap[GetDest[m],GetDest[m]]]

(* Returns the source of \[Theta], the twist transformation, from input of m, the monodial functor 
(really, since \[Theta] goes from id to id, it uses m to find out what the base object is of the monoidal category and the details of m don't matter) *)
Get\[Theta]Source[m_]:=GetIdentityOneCell[GetDest[m]]

(* Returns the destination of \[Theta], the twist transformation, from input of m, the monodial functor 
(really, since \[Theta] goes from id to id, it uses m to find out what the base object is of the monoidal category and the details of m don't matter) *)
Get\[Theta]Dest[m_]:=GetIdentityOneCell[GetDest[m]]

(* Returns a completely variable \[Beta] (with the correct OneCell source and destination) for a given m *)
GenerateVariable\[Beta][m_]:=GenerateVariableTwoCell[Generate\[Beta]Source[m],Generate\[Beta]Dest[m]]

(* Returns a completely variable \[Theta] (with the correct OneCell source and destination) for a given m. 
Note: this one is actually trivial as it is the identity TwoCell on the based category*)
GetVariable\[Theta][m_]:=GenerateVariableTwoCell[Get\[Theta]Source[m],Get\[Theta]Dest[m]]


(*Create the context of the current modular tensor category and check things about it*)

(*is the MTC set?*)
CatSet=False;

(*set the MTC*)
SetMTC[m_,u_]:=Module[{},

	Privatem = m;
	Privateu = u;
	
	Private\[Alpha] = GenerateVariable\[Alpha][m];
	Private\[Rho] = GenerateVariable\[Rho][m,u];
	Private\[Lambda] = GenerateVariable\[Lambda][m,u];

	Private\[Beta] = GenerateVariable\[Beta][m];
	Private\[Theta] = GetVariable\[Theta][m];

	CatSet=True;

	PrintMTC[];

	Print["Please set the variables of the form 'a#' with the appropriate value for the MTC you want to work with or leave them as variable and use other functions (i.e. GetPentagonEquation[]) to display the different axiom equations."];
	Print["Please run IsMTC[] to check that the structures you entered satisfy the MTC axioms."]
]

(*set MTC with specific structures*)
SetMTC[m_,u_,\[Alpha]Set_,\[Rho]Set_,\[Lambda]Set_,\[Beta]Set_,\[Theta]Set_]:=Module[{},

	Privatem = m;
	Privateu = u;

	Private\[Alpha] = GenerateVariable\[Alpha][m];
	Private\[Rho] = GenerateVariable\[Rho][m,u];
	Private\[Lambda] = GenerateVariable\[Lambda][m,u];

	Private\[Beta] = GenerateVariable\[Beta][m];
	Private\[Theta] = GetVariable\[Theta][m];

	CatSet=True;
	
	Set\[Alpha][\[Alpha]Set];
	Set\[Beta][\[Beta]Set];
	Set\[Theta][\[Theta]Set];
	Set\[Lambda][\[Lambda]Set];
	Set\[Rho][\[Rho]Set];

	PrintMTC[];

	Print["Please run IsMTC[] to check that the structures you entered satisfy the MTC axioms."]

]

(*Sets the MTC to the Fibonacci MTC*)
SetMTCFib[]:=Module[{},
	
	Privatem = {{{1,0,0,1},{0,1,1,1}},{4->2}};
	Privateu = {{{1},{0}},{1->2}};
	
	Private\[Alpha] = {{{{{1}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{{1}},{{1}}},{{-1,{0->0}},{{1}},{{1}},{{1}},{{1}},{{1}},{{1}},{{1/GoldenRatio,1/Sqrt[GoldenRatio]},{1/Sqrt[GoldenRatio],-1/GoldenRatio}}}},{{{{1,0,0,1,0,1,1,1},{0,1,1,1,1,1,1,2}},{8->2}}->{{{1,0,0,1,0,1,1,1},{0,1,1,1,1,1,1,2}},{8->2}}}};
	Private\[Rho] = {{{{{1}},{-1,{0->0}}},{{-1,{0->0}},{{1}}}},{{{{1,0},{0,1}},{2->2}}->{{{1,0},{0,1}},{2->2}}}};
	Private\[Lambda] = {{{{{1}},{-1,{0->0}}},{{-1,{0->0}},{{1}}}},{{{{1,0},{0,1}},{2->2}}->{{{1,0},{0,1}},{2->2}}}};

	Private\[Beta] = {{{{{1}},{-1,{0->0}},{-1,{0->0}},{{Exp[-4*Pi*I/5]}}},{{-1,{0->0}},{{1}},{{1}},{{Exp[3*Pi*I/5]}}}},{{{{1,0,0,1},{0,1,1,1}},{4->2}}->{{{1,0,0,1},{0,1,1,1}},{4->2}}}};
	Private\[Theta] = {{{{{1}},{-1,{0->0}}},{{-1,{0->0}},{{Exp[I*Pi*4/5]}}}},{{{{1,0},{0,1}},{2->2}}->{{{1,0},{0,1}},{2->2}}}};

	CatSet=True;
	PrintMTC[];

	Print["The nontrivial simple object as a 1-Cell:"] 
	Print["\[Tau] = ", {{{0},{1}},{1->2}}//OneCellForm];
]

(*Sets the MTC to the Ising MTC*)
SetMTCIsing[]:=Module[{},
	
	Privatem = {{{1,0,0,0,1,0,0,0,1},{0,1,0,1,0,1,0,1,0},{0,0,1,0,1,0,1,0,0}},{9->3}};
	Privateu = {{{1},{0},{0}},{1->3}};
	
	Private\[Alpha] = {{{{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}}},{{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1/Sqrt[2],1/Sqrt[2]},{1/Sqrt[2],-1/Sqrt[2]}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{-1}},{-1,{0->0}},{{1}},{-1,{0->0}}},{{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{-1}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{1}}}},{{{{1,0,0,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,1,0,1,0,0},{0,1,0,1,0,1,0,1,0,1,0,1,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0},{0,0,1,0,1,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,0,0,1}},{27->3}}->{{{1,0,0,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,1,0,1,0,0},{0,1,0,1,0,1,0,1,0,1,0,1,0,2,0,1,0,1,0,1,0,1,0,1,0,1,0},{0,0,1,0,1,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,0,0,1}},{27->3}}}};
	Private\[Rho] = {{{{{1}},{-1,{0->0}},{-1,{0->0}}},{{-1,{0->0}},{{1}},{-1,{0->0}}},{{-1,{0->0}},{-1,{0->0}},{{1}}}},{{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}->{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}}};
	Private\[Lambda] = {{{{{1}},{-1,{0->0}},{-1,{0->0}}},{{-1,{0->0}},{{1}},{-1,{0->0}}},{{-1,{0->0}},{-1,{0->0}},{{1}}}},{{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}->{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}}};

	Private\[Beta] = {{{{{1}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{Exp[-I*Pi/8]}},{-1,{0->0}},{-1,{0->0}},{-1,{0->0}},{{-1}}},{{-1,{0->0}},{{1}},{-1,{0->0}},{{1}},{-1,{0->0}},{{-I}},{-1,{0->0}},{{-I}},{-1,{0->0}}},{{-1,{0->0}},{-1,{0->0}},{{1}},{-1,{0->0}},{{Exp[3*I*Pi/8]}},{-1,{0->0}},{{1}},{-1,{0->0}},{-1,{0->0}}}},{{{{1,0,0,0,1,0,0,0,1},{0,1,0,1,0,1,0,1,0},{0,0,1,0,1,0,1,0,0}},{9->3}}->{{{1,0,0,0,1,0,0,0,1},{0,1,0,1,0,1,0,1,0},{0,0,1,0,1,0,1,0,0}},{9->3}}}};
	Private\[Theta] = {{{{{1}},{-1,{0->0}},{-1,{0->0}}},{{-1,{0->0}},{{Exp[Pi*I/8]}},{-1,{0->0}}},{{-1,{0->0}},{-1,{0->0}},{{-1}}}},{{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}->{{{1,0,0},{0,1,0},{0,0,1}},{3->3}}}};

	CatSet=True;
	PrintMTC[];

	Print["The nontrivial simple objects as 1-Cells:"] 
	Print["\[Sigma] = ", {{{0},{1},{0}},{1->3}}//OneCellForm];
	Print["\[Psi] = ", {{{0},{0},{1}},{1->3}}//OneCellForm];
]

(*sets the various MTC individual structures*)
Set\[Alpha][\[Alpha]Set_]:= If[CatSet,
	If[GetSource[Private\[Alpha]]==GetSource[\[Alpha]Set]&&GetDest[Private\[Alpha]]==GetDest[\[Alpha]Set],
		Private\[Alpha]=\[Alpha]Set,
		Print["Input 2-Cell \[Alpha] is not of the correct form."],	
		Print["Input 2-Cell \[Alpha] is not of the correct form."]
	]
	, 
	Print["The current MTC is not set. Please set the fusion rules and the unit first by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

Set\[Rho][\[Rho]Set_]:= If[CatSet,
	If[GetSource[Private\[Rho]]==GetSource[\[Rho]Set]&&GetDest[Private\[Rho]]==GetDest[\[Rho]Set],
		Private\[Rho]=\[Rho]Set,
		Print["Input 2-Cell \[Rho] is not of the correct form."],	
		Print["Input 2-Cell \[Rho] is not of the correct form."]
	]
	, 
	Print["The current MTC is not set. Please set the fusion rules and the unit first by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

Set\[Lambda][\[Lambda]Set_]:= If[CatSet,
	If[GetSource[Private\[Lambda]]==GetSource[\[Lambda]Set]&&GetDest[Private\[Lambda]]==GetDest[\[Lambda]Set],
		Private\[Lambda]=\[Lambda]Set,
		Print["Input 2-Cell \[Lambda] is not of the correct form."],	
		Print["Input 2-Cell \[Lambda] is not of the correct form."]
	]
	, 
	Print["The current MTC is not set. Please set the fusion rules and the unit first by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

Set\[Beta][\[Beta]Set_]:= If[CatSet,
	If[GetSource[Private\[Beta]]==GetSource[\[Beta]Set]&&GetDest[Private\[Beta]]==GetDest[\[Beta]Set],
		Private\[Beta]=\[Beta]Set,
		Print["Input 2-Cell \[Beta] is not of the correct form."],	
		Print["Input 2-Cell \[Beta] is not of the correct form."]
	]
	, 
	Print["The current MTC is not set. Please set the fusion rules and the unit first by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

Set\[Theta][\[Theta]Set_]:= If[CatSet,
	If[GetSource[Private\[Theta]]==GetSource[\[Theta]Set]&&GetDest[Private\[Theta]]==GetDest[\[Theta]Set],
		Private\[Theta]=\[Theta]Set,
		Print["Input 2-Cell \[Theta] is not of the correct form."],	
		Print["Input 2-Cell \[Theta] is not of the correct form."]
	]
	, 
	Print["The current MTC is not set. Please set the fusion rules and the unit first by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

(*print all the structures of the MTC*)
PrintMTC[]:=Module[{},

	If[CatSet,
		Print["m = ",Privatem//OneCellForm];
		Print["u = ",Privateu//OneCellForm];

		Print["\[Alpha] = ",Private\[Alpha]//TwoCellForm];
		Print["\[Beta] = ",Private\[Beta]//TwoCellForm];
		Print["\[Theta] = ",Private\[Theta]//TwoCellForm];
		Print["\[Rho] = ",Private\[Rho]//TwoCellForm];
		Print["\[Lambda] = ",Private\[Lambda]//TwoCellForm];
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

(*determines if the loaded MTC satisfies all the axioms*)
IsMTC[]:=
If[CatSet, 
	IsMTC[Privatem,Private\[Alpha],Private\[Rho],Private\[Lambda],Privateu,Private\[Beta],Private\[Theta]],
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]

(*returns the various MTC structures*)
Getm[]:= If[CatSet,Privatem, Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Getu[]:= If[CatSet,Privateu, Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Get\[Alpha][]:= If[CatSet,Private\[Alpha], Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Get\[Rho][]:= If[CatSet,Private\[Rho], Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Get\[Lambda][]:= If[CatSet,Private\[Lambda], Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Get\[Beta][]:= If[CatSet,Private\[Beta], Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]
Get\[Theta][]:= If[CatSet,Private\[Theta], Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]]

(*returns MTC equations in various forms*)
GetPentagonEquation[DisplayForm_:True]:=If[CatSet,
	IsPentagon[Privatem,Private\[Alpha],False,DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetTriangleEquation[DisplayForm_:True]:=If[CatSet,
	IsTriangle[Privatem,Private\[Alpha],Private\[Rho],Private\[Lambda],Privateu,False,DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetHexagonEquation1[DisplayForm_:True]:=If[CatSet,
	IsHexagon1[Privatem,Private\[Alpha],Private\[Beta],DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetHexagonEquation2[DisplayForm_:True]:=If[CatSet,
	IsHexagon2[Privatem,Private\[Alpha],Private\[Beta],DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetTwistEquation[DisplayForm_:True]:=If[CatSet,
	IsTwisted[Privatem,Private\[Beta],Private\[Theta],False,DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetRibbonCoherenceEquation[DisplayForm_:True]:=If[CatSet,
	IsRibbonCoherent[Privatem,Private\[Theta],Privateu,DisplayForm]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

(*Generate the handle of the MTC*)
GenerateHandle[]:=If[CatSet,
	GenerateHandle[Privatem,Privateu]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]


(*monoidal combination within the modular tensor category. if both objects are 1-Cells,
then it monoidally combines the objects. if both objects are 2-cells, then it monodially
combines the morphisms*)
SetAttributes[CircleTimes,{Flat,OneIdentity}]
CircleTimes[A_,B_]:=Module[{TypeA,TypeB},
	If[CatSet,
			Privatem\[SmallCircle](A\[CircleDot]B)
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]
CircleTimes[A_,B_,C_]:=Module[{TypeAB,TypeC,AB},
	If[CatSet,
		AB=CircleTimes[A,B];
		TypeAB=GetCellType[AB];
		TypeC=GetCellType[C];

		If[TypeAB==1&&TypeC==1,
			Privatem\[SmallCircle](AB\[CircleDot]C),
			Print["Operation is not associative. Please use appropriate bracketing."]
		]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

(*Automatically reboxed functions for when the 2-cell is to be applied to specific objects, within the current category*)
Unprotect[\[Alpha],\[Beta],\[Theta],\[Lambda],\[Rho]]

\[Alpha][A_,B_,C_]:= Module[{},
	If[CatSet,
		\[Alpha]ABC[Privatem,Private\[Alpha],A,B,C]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

\[Beta][A_,B_]:= Module[{},
	If[CatSet,
		\[Beta]AB[Privatem,Private\[Beta],A,B]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

\[Theta][A_]:= Module[{},
	If[CatSet,
		\[Theta]A[Privatem,Private\[Theta],A]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

\[Lambda][A_]:= Module[{},
	If[CatSet,
		\[Lambda]A[Privatem,Private\[Lambda],Privateu,A]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

\[Rho][A_]:= Module[{},
	If[CatSet,
		\[Rho]A[Privatem,Private\[Rho],Privateu,A]
		,
		Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
	]
]

Protect[\[Alpha],\[Beta],\[Theta],\[Lambda],\[Rho]]


(*Monoid equations*)

(*test if a monoid on the currently loaded MTC satisfies the monoid axioms*)
IsMonoid[A_,\[Mu]_,\[Eta]_]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			If[GetCellType[\[Eta]]!=2,
				Print["Error: \[Eta], the monoid unit, must be a valid 2-Cell."]
				,
				IsMonoid[Privatem,Privateu,Private\[Alpha],Private\[Rho],Private\[Lambda],A,\[Mu],\[Eta]]
			]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

(*test if a monoid on the currently loaded MTC satisfies the dagger frobenious axioms*)
IsMonoidDaggerFrobenius[A_,\[Mu]_,\[Eta]_]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			If[GetCellType[\[Eta]]!=2,
				Print["Error: \[Eta], the monoid unit, must be a valid 2-Cell."]
				,
				IsMonoidDaggerFrobenius[Privatem,Private\[Alpha],A,\[Mu],\[Eta]]
			]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

(*tests if a monoid on the currently loaded MTC is commutative*)
IsMonoidCommutative[A_,\[Mu]_]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			IsMonoidCommutative[Privatem,A,\[Mu],Private\[Beta]]		
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]


(*return and/or check equations in various form*)
GetMonoidAssociativityEquation[A_,\[Mu]_,DisplayForm_:True]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			IsMonoidAssociative[Privatem,A,\[Mu],Private\[Alpha],DisplayForm]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetMonoidRightUnitEquation[A_,\[Mu]_,\[Eta]_,DisplayForm_:True]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			If[GetCellType[\[Eta]]!=2,
				Print["Error: \[Eta], the monoid unit, must be a valid 2-Cell."]
				,
				IsMonoidRightUnit[Privatem,A,\[Mu],\[Eta],Privateu,Private\[Rho],DisplayForm]
			]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetMonoidLeftUnitEquation[A_,\[Mu]_,\[Eta]_,DisplayForm_:True]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			If[GetCellType[\[Eta]]!=2,
				Print["Error: \[Eta], the monoid unit, must be a valid 2-Cell."]
				,
				IsMonoidLeftUnit[Privatem,A,\[Mu],\[Eta],Privateu,Private\[Lambda],DisplayForm]
			]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetMonoidDaggerFrobeniusEquation[A_,\[Mu]_,\[Eta]_,DisplayForm_:True]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			If[GetCellType[\[Eta]]!=2,
				Print["Error: \[Eta], the monoid unit, must be a valid 2-Cell."]
				,
				IsMonoidDaggerFrobenius[Privatem,Private\[Alpha],A,\[Mu],\[Eta],DisplayForm]
			]
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]

GetMonoidCommutativityEquation[A_,\[Mu]_,DisplayForm_:True]:=If[CatSet,
	If[GetCellType[A]!=1,
		Print["Error: A, the prospective monoid, must be a valid 1-Cell object."]
		,
		If[GetCellType[\[Mu]]!=2,
			Print["Error: \[Mu], the monoid multplication, must be a valid 2-Cell."]
			,
			IsMonoidCommutative[Privatem,A,\[Mu],Private\[Beta],DisplayForm]		
		]
	]
	,
	Print["The current MTC is not set. Please set it by calling SetMTC[m,u] where m is the 1-Cell fusion rules, and u is the 1-Cell unit."]
]



End[ ]

EndPackage[ ]

