(* ::Package:: *)

(*

A Mathematica Package for 2Vect and Modular Tensor Categories

Daniel A. Roberts
Department of Computer Science
University of Oxford

TwoVect.m

*)

BeginPackage["TwoVect`"]

Dim::usage =
        "Dim[x] gives the dimensions of the matrix x and can handle 0D matrices."

Is0D::usage =
		"Is0D[A] determine whether the matrix (not a 1-Cell or a 2-Cell) A is in 0D form."

TensorProduct::usage =
		"TensorProduct[x,y] gives the tensor product of matrices x and y. 0D matrices should be entered in the form {-1,{n\[Rule]m}}, where at least one of m or n must be a 0. 1 x n dimensional matrices must be entered in the form {{a,b,c, ...}}, and n x 1 dimensional matrices must be entered in the form {{a},{b},{c},...}."

DirectSum::usage =
		"DirectSum[x,y] gives the direct sum of matrices x and y. 0D matrices should be entered in the form {-1,{n\[Rule]m}}, where at least one of m or n must be a 0. 1 x n dimensional matrices must be entered in the form {{a,b,c, ...}}, and n x 1 dimensional matrices must be entered in the form {{a},{b},{c},...}."

Dot0D::usage =
		"Dot0D[A,B] generates a matrix dot product, AB, that works with 0D matrices."

IdentityMat::usage =
		"IdentityMat[n] gives the n x n identity matrix and can handle n=0."

MatrixInverse::usage = 
		"MatrixInverse[A] if it exists, computes A^{-1}, the inverse of the matrix A, and can handle 0D matrices."
 
IsMatrixInvertible::usage =
		"IsMatrixInvertible[A] returns True if the matrix A is invertible and False if it is not. Only square matrices are candidates to be invertible. Handles 0D matrices.."

GetAdjointMatrix::usage =
		"GetAdjointMatrix[A] returns A\[ConjugateTranspose], the adjoint (conjugate transpose) of the input matrix, A. Handles 0D matrices."

GenerateVariableMatrix::usage =
		"GenerateVariableMatrix[InputDim] generates a matrix (with dimensions specified by the list 'InputDim') filled entirely with new unique variables (i.e. to generate an m x n dimensional matrix with variable entries, InputDim={m,n}). Will also work for m=0 and/or n=0."

CellByCellMatrixEquation::usage =
		"CellByCellMatrixEquation[A,B] takes two matrices to be equated and equates them cell by cell (i.e. it creates a matrix of A[[i,j]]==B[[i,j]] rather than just A==B)."

CellByCellTwoMatrixEquation::usage =
		"CellByCellTwoMatrixEquation[A,B] takes two 2-matrices to be equated and equates them element-matrix cell by cell (i.e. it creates a 2-matrix of A[[i,j]][[k,m]]==B[[i,j]][[k,m]] rather than just A==B)."

GetSource::usage =
		"GetSource[A] returns the source n-Cell for either 2-Cells or 1-Cells."

GetDest::usage = 
		"GetDest[A] returns the destination n-Cell for either 2-Cells or 1-Cells."

GetCellType::usage =
		"GetCellType[A] returns 0 if A is a 0-Cell, 1 if A is a 1-Cell, 2 if A is a 2-Cell, and an error otherwise (although if something is neither it may not always catch the error)."

OneCell::usage =
		"OneCell[A] takes a matrix A and converts it into a 1-Cell."

OneToMat::usage =
		"OneToMat[A] takes a 1-Cell A and converts it into a matrix."

OneCellComposition::usage =
		"OneCellComposition[f,g] composes the 1-Cells f and g as f\[SmallCircle]g, with the convention of right to left (i.e. g followed by f)."

OneCellTimes::usage =
		"OneCellTimes[f,g] combines the 1-Cells f and g in parallel as f[x]g using the tensor product 2-functor [x]."

GetIdentityOneCell::usage =
		"GetIdentityOneCell[n] returns the identity 1-Cell of n, the input 0-Cell."

OneCellInverse::usage =
		"OneCellInverse[f] takes the input 1-Cell f and returns the inverse 1-Cell, if it exists."

IsOneCellInvertible::usage =
		"IsOneCellInvertible[f] returns True if the 1-Cell f is invertible and False if it is not. Only 1-Cells that go between the same 0-Cells (i.e. represented by square matrices) are candidates to be invertible."

GetAdjointOneCell::usage =
		"GetAdjointOneCell[A] returns the adjoint (conjugate transpose) of the input 1-Cell."

GetOneCellSwap::usage =
		"GetOneCellSwap[m,n] will return the 1-Cell S_{m,n}. If 0-Cells m and n (i.e. 2 natural numbers) are combined using the tensor product 2-functor as m [x] n, then S_{m,n} is the swap 1-Cell such that S_{m,n} acting on m [x] n will give n [x] m."

TwoCell::usage =
		"TwoCell[A] takes a 2-matrix (i.e. a matrix of matrices) A and converts it into a 2-Cell."

TwoToTwoMat::usage =
		"TwoToTwoMat[A] takes a 2-Cell A and converts it into a 2-matrix (i.e. a matrix of matrices)."

TwoCellVerticalComposition::usage =
		"TwoCellVerticalComposition[F,G] composes the 2-Cells F and G vertically as F\[SmallCircle]G, with the convention of right to left (i.e. G followed by F). The source of F must equal the destination of G."

TwoCellHorizontalComposition::usage =
		"TwoCellHorizontalComposition[G,F] composes the 2-Cells G and F horizontally as G\[CenterDot]F, with the convention of right to left (i.e. F followed by G). The destination 0-Cell of the source and destination 1-Cells of F must be the same as the source 0-Cell of the source and destination 1-Cells of G."

TwoCellTimes::usage =
			"TwoCellTimes[F,G] combines the 2-Cells F and G in parallel as F[x]G using the tensor product 2-functor [x]."

GetIdentityTwoCell::usage =
		"GetIdentityTwoCell[f] returns the identity 2-Cell (under vertical composition) of f, the input 1-Cell."

GetIdentity::usage =
		"GetIdentity[X] returns the identity n+1-Cell of the n-Cell X."

TwoCellInverse::usage =
		"TwoCellInverse[F] takes the input 2-Cell F and returns the inverse 2-Cell (under vertical composition), if it exists."

IsTwoCellInvertible::usage =
		"IsTwoCellInvertible[F] returns True if the 2-Cell F is invertible (under vertical composition) and False if it is not. Only 2-Cells whose element-matrices (i.e. inner matrices) are all square are candidates to be invertible."

GetAdjointTwoCell::usage =
		"GetAdjointTwoCell[A] returns the adjoint (conjugate transpose) of the input 2-Cell."

WhiskerRight::usage =
		"WhiskerRight[f,G], for input 1-Cell f and 2-Cell G, horizontally combines the 2-Cell G followed by the 2-Cell given by GetIdentityTwoCell[f] (i.e. the 2-Cell id_f\[CenterDot]G). The order is a little unintuitive."

WhiskerLeft::usage =
		"WhiskerLeft[G,f], for input 2-Cell G and 1-Cell f, horizontally combines the 2-Cell given by GetIdentityTwoCell[f] followed by the 2-Cell G (i.e. the 2-Cell G\[CenterDot]id_f). The order is a little unintuitive."

GenerateVariableTwoCell::usage =
		"GenerateVariableTwoCell[TwoCellSource,TwoCellDest] returns a 2-Cell whose  element-matrix elements (i.e. the elements of the inner matrices) are filled entirely with new and unique variables. The inputs TwoCellSource and TwoCellDest are the 1-Cell source and destination, respectively, of the new 2-Cell to be generated."

Get\[Omega]::usage =
	"Get\[Omega][a,b,c], with 1-Cell inputs a, b, and c, is used to generate the structural isomorphism horizontal associator, \[Omega], which is used to correct for horizontally composing three 2-Cells in a row. If A.(B.C) is taken to mean horizontally composing (B.C) and then composing with A, (with the understanding that the order of composition is C, then B, then A), then Get\[Omega][GetDest[A],GetDest[B],GetDest[C]]*(A.(B.C)) \[Equal] ((A.B).C))*Get\[Omega][GetSource[A],GetSource[B],GetSource[C]] with \[Omega]*(A.(B.C)) taken to mean vertical composition of \[Omega] with (A.(B.C)) (again, in the order (A.(B.C)) followed by \[Omega])."

Get\[Tau]::usage =
	"Get\[Tau][a,b,c,d], with 1-Cell inputs a, b, c, and d, is used to generate the structural isomorphism, \[Tau], which is used to correct for the fact that horizontally composing two pairs of 2-Cells and then monoidally combining them is not the same as monoidally combining the two pairs and then horizontally composing them. If (A.B)[x](C.D) is taken to mean horizontally composing (A.B) and (C.D) and then monoidally combining them, then Get\[Tau][GetDest[A],GetDest[B],GetDest[C],GetDest[D]]*(A.B)[x](C.D) == (A[x]C).(B[x]D)*Get\[Tau][GetSource[A],GetSource[B],GetSource[C],GetSource[D]], with the '*' operation taken to mean vertical composition."

Get\[Sigma]::usage =
	"Get\[Sigma][A,B], with 1-Cell inputs A and B, is used to generate the structural isomorphism, \[Sigma], a 2-Cell swap (i.e. an interchange between a 1-Cell swap and 1-Cells on each leg of the swap). Thus, at the 2-Cell level, if * means vertical composition and . means horizontal composition, and \[Mu]: F1->F2 and \[Nu]:G1->G2, then Get\[Sigma][F2,G2]*(idS_{B,D}.(\[Mu][x]\[Nu]) ) == ( (\[Nu][x]\[Mu]).idS_{B,D} )*Get\[Sigma][F1,G1]."

\[Omega]::usage =
	"Same as Get\[Omega][a,b,c]. \[Omega][a,b,c], with 1-Cell inputs a, b, and c, is used to generate the structural isomorphism horizontal associator, \[Omega], which is used to correct for horizontally composing three 2-Cells in a row. If A.(B.C) is taken to mean horizontally composing (B.C) and then composing with A, (with the understanding that the order of composition is C, then B, then A), then Get\[Omega][GetDest[A],GetDest[B],GetDest[C]]*(A.(B.C)) \[Equal] ((A.B).C))*Get\[Omega][GetSource[A],GetSource[B],GetSource[C]] with \[Omega]*(A.(B.C)) taken to mean vertical composition of \[Omega] with (A.(B.C)) (again, in the order (A.(B.C)) followed by \[Omega])."

\[Tau]::usage =
	"Same as Get\[Tau][a,b,c,d]. \[Tau][a,b,c,d], with 1-Cell inputs a, b, c, and d, is used to generate the structural isomorphism, \[Tau], which is used to correct for the fact that horizontally composing two pairs of 2-Cells and then monoidally combining them is not the same as monoidally combining the two pairs and then horizontally composing them. If (A.B)[x](C.D) is taken to mean horizontally composing (A.B) and (C.D) and then monoidally combining them, then Get\[Tau][GetDest[A],GetDest[B],GetDest[C],GetDest[D]]*(A.B)[x](C.D) == (A[x]C).(B[x]D)*Get\[Tau][GetSource[A],GetSource[B],GetSource[C],GetSource[D]], with the '*' operation taken to mean vertical composition."

\[Sigma]::usage =
	"Same as Get\[Sigma][A,B]. \[Sigma][A,B], with 1-Cell inputs A and B, is used to generate the structural isomorphism, \[Sigma], a 2-Cell swap (i.e. an interchange between a 1-Cell swap and 1-Cells on each leg of the swap). Thus, at the 2-Cell level, if * means vertical composition and . means horizontal composition, and \[Mu]: F1->F2 and \[Nu]:G1->G2, then Get\[Sigma][F2,G2]*(idS_{B,D}.(\[Mu][x]\[Nu]) ) == ( (\[Nu][x]\[Mu]).idS_{B,D} )*Get\[Sigma][F1,G1]."

TwoMatrixForm::usage =
		"TwoMatrixForm[F] display 2-matrices (i.e. matrices of matrices, or, 2-Cells without their 1-Cell source and destination) in a form equivalent to MatrixForm[F] for regular matrices."

OneCellForm::usage = 
		"OneCellForm[f] display 1-Cells in a form equivalent to MatrixForm[F] for regular matrices."

TwoCellForm::usage = 
		"TwoCellForm[F] display 2-Cells in a form equivalent to MatrixForm[F] for regular matrices."

SmallCircle::usage =
		"A\[SmallCircle]B\[SmallCircle]... or SmallCircle[A,B,...] computes 1-Cell composition or 2-Cell vertical composition, depending on the cell type of the arguments. This is an n-ary operation as composition is associative."

CircleDot::usage =
		"A\[CircleDot]B\[CircleDot]... or CircleDot[A,B,...] computes 1-Cell parallel combination or 2-Cell parallel combination, in 2Vect, depending on the cell type of the arguments. This is an n-ary operation as the tensor product is associative. Note: this operation is usually represented by [x] and is the tensor product 2-functor."

CenterDot::usage =
		"A\[CenterDot]B or CenterDot[A,B] computes 2-Cell horizontal composition. Arguments can be 1-Cells or 2-Cells, as the 1-Cells are whiskered as appropriate (i.e. they are converted to 2-Cells by the function GetIdentityTwoCell). Note: this is only a binary operation as horizontal composition is not associative."

id::usage =
		"id[A] returns either the identity 1-Cell or the identity 2-Cell, depending on whether the argument is a 0-Cell or 1-Cell, respectively."

inv::usage =
		"inv[A] returns either the inverse 1-Cell or the inverse 2-Cell, depending on whether the argument is a 1-Cell or 2-Cell, respectively."

Compose2Cells::usage =
		"Compose2Cells[list] vertically composes a list of 2-Cells. The first element of the list is the first 2-Cell applied, which is usually written as the right-most 2-Cell (i.e. if A and B are 2-Cells and you want to compute A\[SmallCircle]B, then you would enter {B,A} as the argument to this function). This is an alternative to the many ways of vertically composing 2-Cells."


Begin["`Private`"]


(*Generalized Operations*)

(* Operates on Matrices like matrix multiplication, but lets the user specify the "Plus" and "Times" operations *)
(*Note: will not work for 0d.*)
GeneralizedDot[t_,s_,GeneralizedPlus_,GeneralizedTimes_]:=
Module[{i,j,k,ans},

	If[Dimensions[t][[2]]!=Dimensions[s][[1]],Print["Error: Cannot compute GeneralizedDot. The matrices are not compatible."],

		ans=ConstantArray[0,{Dimensions[t][[1]],Dimensions[s][[2]]}];
		For[i=1,i<=Dimensions[t][[1]],i++,

			For[j=1,j<=Dimensions[s][[2]],j++,

				ans[[i,j]]=GeneralizedTimes[ t[[i]][[1]],s[[1]][[j]] ];
			
				For[k=2,k<= Dimensions[t][[2]],k++,
					ans[[i,j]] = GeneralizedPlus[ans[[i,j]],GeneralizedTimes[t[[i]][[k]],s[[k]][[j]]]];
				]

			]
		];

		Return[ans];
	]
]

(* Operates on Matrices element by element, but lets users specify the operation *)
(*Note: will not work for 0d.*)
GeneralizedBinaryElementOperation[t_,s_,GeneralizedOperation_]:=
Module[{i,j,ans},

	If[Dimensions[t][[1]]!=Dimensions[s][[1]]||Dimensions[t][[2]]!=Dimensions[s][[2]],Print["Error: Cannot compute GeneralizedBinaryElementOperation. The matrices are not compatible."],

		ans=ConstantArray[0,{Dimensions[t][[1]],Dimensions[t][[2]]}];
		For[i=1,i<=Dimensions[t][[1]],i++,

			For[j=1,j<=Dimensions[t][[2]],j++,
				ans[[i,j]]=GeneralizedOperation[t[[i,j]],s[[i,j]]];
			]
		];
		Return[ans];
	]
]

(* Operates on a Matrix element by element, but lets users specify the operation *)
(*Note: will not work for 0d.*)
GeneralizedUnaryElementOperation[t_,GeneralizedOperation_]:=
Module[{i,j,ans},
	ans=ConstantArray[0,{Dimensions[t][[1]],Dimensions[t][[2]]}];
	For[i=1,i<=Dimensions[t][[1]],i++,

		For[j=1,j<=Dimensions[t][[2]],j++,
			ans[[i,j]]=GeneralizedOperation[t[[i,j]]];
		]
	];
	Return[ans];
]

(*Operates on Matrices like the tensor product, but lets the user specify what the operation is between elements (i.e. for the normal tensor product, it is multiplication) *)
GeneralizedTensorProduct[A_,B_,GeneralizedTimesOperation_]:=Module[{i,j,m,n,ans,rowA,colA,rowB,colB,row,col},

	If[Is0D[A]||Is0D[B],Print["Error: Cannot GeneralizedTensorProduct. At least one of the matrices are 0D."],

		rowA=Dim[A][[1]];
		colA=Dim[A][[2]];
		rowB=Dim[B][[1]];
		colB=Dim[B][[2]];

		ans=ConstantArray[0,{rowA*rowB,colA*colB}];

		For[i=1,i<=rowA,i++,
			For[j=1,j<=colA,j++,
				For[m=1,m<=rowB,m++,
					For[n=1,n<=colB,n++,

						row=(i-1)*rowB+m;
						col=(j-1)colB +n;

						ans[[row,col]]=GeneralizedTimesOperation[A[[i,j]],B[[m,n]] ];

					]
				]
			]
		];

		ans
	]
]


(*2Vect Operations*)

Unprotect[\[EmptySet]];
\[EmptySet]=-1;
Protect[\[EmptySet]];

(*Matrix Functions*)

(* Define a suitable dimension function *)
Dim[x_]:= 
If[x[[1]]==\[EmptySet],
	{x[[2]][[1]][[2]],x[[2]][[1]][[1]]},
	Dimensions[x],
	Dimensions[x]
]

(* Determine whether a MATRIX (not a OneCell or a TwoCell) is in 0D form *)
Is0D[A_]:=(Dim[A][[1]]==0)||(Dim[A][[2]]==0)

(* Define the tensor product *)
(* 0 dimensional matrices should be entered in the form {-1,{n->m}},
where at least one of m or n must be a 0 *)
(* n x 1 matrices must be entered in the form {{a,b,c, ...}} *)
TensorProduct[x_,y_]:=
If[(x[[1]]==\[EmptySet])||(y[[1]]==\[EmptySet] ),
	{\[EmptySet],{(Dim[x]*Dim[y])[[2]]-> (Dim[x]*Dim[y])[[1]]}},
	KroneckerProduct[x,y],
	KroneckerProduct[x,y]
]

(*Define the Direct Sum*)
DirectSumNo0[x_,y_]:=ArrayFlatten[{{x, 0},{0,y}}]

(*Define the 0D safe Direct Sum*)
DirectSum[x_,y_]:=
If[(Is0D[x]||Is0D[y])&&!(Is0D[x]&&Is0D[y]),
	If[Is0D[x],
		If[Dim[x][[1]]==0,
			ArrayFlatten[{{ConstantArray[0,{Dim[y][[1]],Dim[x][[2]]}],y}}],
			ArrayFlatten[{{ConstantArray[0,{Dim[x][[1]],Dim[y][[2]]}]},{y}}]
		],
		If[Dim[y][[1]]==0,
			ArrayFlatten[{{x,ConstantArray[0,{Dim[x][[1]],Dim[y][[2]]}]}}],
			ArrayFlatten[{{x},{ConstantArray[0,{Dim[y][[1]],Dim[x][[2]]}]}}]
		]
	],
	If[(Is0D[x]&&Is0D[y]),
		If[((Dim[x][[1]]+Dim[y][[1]]) == 0)||((Dim[x][[2]]+Dim[y][[2]])==0),
			{\[EmptySet],{(Dim[x][[2]]+Dim[y][[2]]) ->(Dim[x][[1]]+Dim[y][[1]])}},
			ConstantArray[0,Dim[x]+Dim[y]]
		],
		DirectSumNo0[x,y]
	]
]

(*Define a matrix dot product to work with 0D Matrices *)
Dot0D[A_,B_]:=
If[Dim[A][[2]]==Dim[B][[1]],
	If[(Is0D[A]||Is0D[B]),
		If[(Dim[A][[1]]==0)||(Dim[B][[2]]==0),
			{\[EmptySet],{Dim[B][[2]]->Dim[A][[1]]}},
			ConstantArray[0,{Dim[A][[1]],Dim[B][[2]]}]
		],
		A.B
	],
	Print["Error: Sizes incompatible for matrix multiplication"]
]

(*returns Identity Matrix (works for 0D)*)
IdentityMat[n_]:=
If[n==0,
	{\[EmptySet],{0-> 0}},
	IdentityMatrix[n]
]

(*If Det[A]!=0, the matrix is invertible. For 0D Matrices, only square matrices (both Dimensions are zero) are invertible*) 
IsMatrixInvertible[A_]:=Module[{dimA},
	dimA=Dim[A];
	If[dimA[[1]]!= dimA[[2]],
		False,
		If[dimA[[1]]==0,True,
			Det[A]!= 0
		]
	]
]

(*Computes the inverse of the matrix, if it exists. Safe for 0D matrices *)
MatrixInverse[A_]:=Module[{dimA},
	dimA=Dim[A];
	If[dimA[[1]]!= dimA[[2]],
		Print["Error: Non-square matrix can't be inverted."],
		If[dimA[[1]]==0,
			Return[{\[EmptySet],{0->0}}],
			If[Det[A]==0,
				Print["Error: Matrix can't be inverted."],
				Inverse[A]
			]
		]
	]
]

(* Returns the Adjoint (conjugate transpose) of the input Matrix. Safe for 0d *)
GetAdjointMatrix[A_]:=
If[Is0D[A],
	{\[EmptySet],{Dim[A][[1]]-> Dim[A][[2]]}},
	ConjugateTranspose[A]
]

(* Generates a matrix filled entirely with new unique variables. Safe for 0D matrices *)
GenerateVariableMatrix[InputDim_]:=
If[InputDim[[1]]==0||InputDim[[2]]==0,
	{\[EmptySet],{InputDim[[2]]->InputDim[[1]]}},
	GeneralizedUnaryElementOperation[ConstantArray["a",InputDim],Unique]
]

(*equates all the individual cells of a matrix rather than the whole thing*)
CellByCellMatrixEquation[A_,B_]:=If[Is0D[A]||Is0D[B],{{A==B}},GeneralizedBinaryElementOperation[A,B,Equal]]

(*equates all the individual element-matrix element cells of a 2-matrix rather than the whole thing*)
CellByCellTwoMatrixEquation[A_,B_]:=Module[{i,j,ans},
	If[Dimensions[A][[1]]!=Dimensions[B][[1]]||Dimensions[A][[2]]!=Dimensions[B][[2]],
		Print["Error: Cannot display in equation form. The 2-matrices do not have the same outer dimensions."]
		,
		ans=A;
		For[i=1,i<=Dimensions[A][[1]],i++,
			For[j=1,j<=Dimensions[A][[2]],j++,
				ans[[i,j]]=CellByCellMatrixEquation[A[[i,j]],B[[i,j]]]
			]
		];
		ans
	]
]


(*2Vect Functions for 1-cells and 2-cells*)

(*General Functions*)

(* Get Source and Destination for either TwoCells or OneCells *)
GetSource[A_]:=A[[2]][[1]][[1]]
GetDest[A_]:=A[[2]][[1]][[2]]

(*Return 0 if a 0-cell, 1 if a 1-cell, 2 if a 2-cell, and an error otherwise [although if something is neither it may not always catch the error]*)
GetCellType[A_]:=Module[{},
	If[Depth[A]==1,0,
		If[Depth[A[[2]]]==3,1,
			If[Depth[A[[2]]]==6,2,
				Print["Not a 0-cell, 1-cell, or 2-cell."]
			]
		]
	]
]


(*1-Cell Functions*)

(* Take a Matrix and convert to One Cell *)
OneCell[A_]:= 
If[Is0D[A],
	A,
	{A,{Dim[A][[2]]->Dim[A][[1]]}} 
]

(* Take a OneCell and get a Matrix *)
OneToMat[A_]:=
If[GetSource[A]==0||GetDest[A]==0,
	A,
	A[[1]]
]

(* Define OneCell composition *)
OneCellComposition[f_,g_]:=OneCell[Dot0D[OneToMat[f],OneToMat[g]]]

(* Combine OneCells in parallel, using the Tensor Product *)
OneCellTimes[f_,g_]:=
If[Is0D[f//OneToMat]||Is0D[g//OneToMat],
	TensorProduct[OneToMat[f],OneToMat[g]],
	{TensorProduct[OneToMat[f],OneToMat[g]],{(GetSource[f]*GetSource[g])->(GetDest[f]*GetDest[g])}}
]

(*returns Identity OneCell*)
GetIdentityOneCell[n_]:=
If[n==0,
	{\[EmptySet],{0-> 0}},
	{IdentityMatrix[n],{n->n}}
]

(* Compute the inverse of a OneCell. *)
OneCellInverse[f_]:=MatrixInverse[f//OneToMat]//OneCell

(* Check if OneCell is invertible. Safe for 0D OneCells*)
IsOneCellInvertible[f_]:=IsMatrixInvertible[f//OneToMat]

(* returns the Adjoint of the input OneCell 
(N.B. in this case it doesn't necessarily need to take the conjugate transpose, it could just take the transpose, 
since the entries in OneCells are represented natural numbers, the dimensions of vector spaces. 
Technically, this operation should take the dual of the vector spaces, by this is unnecessary due to how OneCells are represented) *)
GetAdjointOneCell[A_]:=GetAdjointMatrix[A//OneToMat]//OneCell

(* Input 2 objects, m and n (i.e. 2 natural numbers), that are combined as m [x] n, and it will output a Swap OneCell that when applied to m [x] n will give n [x] m *)
GetOneCellSwap[m_,n_]:=Module[{j, i, ans},
	If[m==0||n==0,{\[EmptySet],{0->0}},
		ans=ConstantArray[0,{m*n,m*n}];
		(*j acts as a counter that determines the place in the output column where the input at i will be swaped to*)
		For[i=1,i<=m*n,i++,
			j=1+Mod[((i-1)*n ),m*n]+Quotient[((i-1)*n ),m*n];
			ans[[i,j]]=1
		];
		OneCell[ans]
	]
]


(*2-Cell Functions*)

(* Take a TwoMatrix (a matrix of a matrix) and convert to a TwoCell *)
(* FLAGGED FOR REPLACEMENT BY AN GENERALIZED FUNCTION IMPLEMENTATION *)
TwoCell[A_]:=Module[{i,j,f,g},
	f=ConstantArray[0,{Dim[A][[1]],Dim[A][[2]]}];
	g=ConstantArray[0,{Dim[A][[1]],Dim[A][[2]]}];
	For[i=1,i<= Dim[A][[1]],i++,
		For[j=1,j<=Dim[A][[2]],j++,
			f[[i,j]]=Dim[A[[i,j]]][[2]];
			g[[i,j]]=Dim[A[[i,j]]][[1]];
		]
	];
	{A,{OneCell[f]->OneCell[g]}}
]

(* Take a TwoCell and return a TwoMatrix *)
TwoToTwoMat[A_]:=A[[1]]

(* Define TwoCell Vertical Composition  *) 
(* Two cells are in the form of a list with the first entry being another matrix of the transformation matrices 
and the second entry being OneCell1 -> OneCell2, where the OneCells are in the form used above for one cells *) 
TwoCellVerticalComposition[F_,G_]:={GeneralizedBinaryElementOperation[F[[1]],G[[1]],Dot0D],{G[[2]][[1]][[1]]->F[[2]][[1]][[2]]}}

(* Define TwoCell Horizontal Composition *)
(* As in other compositions, the earlier one goes second, i.e. F followed by G, *)
(* even though that concept doesn't necessarily make sense for horizontal composition *)
TwoCellHorizontalComposition[G_,F_]:=
If[Dim[G[[1]]][[2]]==Dim[F[[1]]][[1]],
	{GeneralizedDot[G[[1]],F[[1]],DirectSum,TensorProduct],{OneCellComposition[ G[[2]][[1]][[1]],F[[2]][[1]][[1]]]->
	OneCellComposition[ G[[2]][[1]][[2]],F[[2]][[1]][[2]]]}},
	Print["Error: Sizes incompatible for Two Cell Horizontal Composition"]
]

(* returns Identity TwoCell for Input OneCell *)
(* not safe for 0D since there is no 0D TwoMatrix, where the outer matrix has a D of 0 *)
GetIdentityTwoCell[f_]:={GeneralizedUnaryElementOperation[f[[1]],IdentityMat],{f->f}}

(* Computes two cell vertical composition inverse such that TwoCellVerticalComposition[TwoCellInverse[F],F]==GetTwoCellIdentity[F] 
and TwoCellVerticalComposition[F,TwoCellInverse[F]]==GetTwoCellIdentity[F]. 
It will print errors about one cells and whatnot if the TwoCell isn't invertible*)
TwoCellInverse[F_]:=GeneralizedUnaryElementOperation[F//TwoToTwoMat,MatrixInverse]//TwoCell

(* Checks if its possible to compute a TwoCell vertical composition inverse  *)
IsTwoCellInvertible[F_]:=
FullSimplify[GeneralizedDot[GeneralizedDot[ConstantArray[True,{1,Dim[F//TwoToTwoMat][[1]]}],
	GeneralizedUnaryElementOperation[F//TwoToTwoMat,IsMatrixInvertible],And,And],
	ConstantArray[True,{Dim[F//TwoToTwoMat][[2]],1}],And,And][[1,1]]]

(* returns the Adjoint of the input TwoCell *)
GetAdjointTwoCell[A_]:=GeneralizedUnaryElementOperation[A//TwoToTwoMat,GetAdjointMatrix]//TwoCell

(* combine two cells in parallel *)
TwoCellTimes[F_,G_]:=GeneralizedTensorProduct[F//TwoToTwoMat,G//TwoToTwoMat,TensorProduct]//TwoCell

(*Whisker functions *)
(* Pay close attention to not intuitive order. Whisker Right is TwoCell  G followed by OneCell f, 
i.e Horizontally combined with the Identity Two Cell that can be derived from f *) 
WhiskerRight[f_,G_]:=TwoCellHorizontalComposition[GetIdentityTwoCell[f],G]
WhiskerLeft[G_,f_]:=TwoCellHorizontalComposition[G,GetIdentityTwoCell[f]]

(* Generates a TwoCell whose entries (in the inner matrices) are filled entirely with new and unique variables. 
N.B.: Assumes source and destination OneCells are compatible (i.e. the OneCells go between the same objects),
although if not an error will be thrown by another function somewhere down the line.*)
GenerateVariableTwoCell[TwoCellSource_,TwoCellDest_]:=Module[{f},
	f[a_,b_]:=GenerateVariableMatrix[{a,b}];
	{GeneralizedBinaryElementOperation[TwoCellDest//OneToMat,TwoCellSource//OneToMat,f],{TwoCellSource->TwoCellDest}}
]

(*returns generic identity 1-cell or 2-cell*)
GetIdentity[n_]:=Module[{type},
	type=GetCellType[n];
	If[type==0,Return[GetIdentityOneCell[n]]];
	If[type==1,Return[GetIdentityTwoCell[n]]];
	If[type==2,Print["Cannot construct identity on a 2-cell"];Return;];	
]


(*2-Cell Structural Isomorphisms*)

(* Get\[Omega] is used to generate horizontal associator \[Omega], (i.e. the structure change of basis map) used to correct for horizontally composing 3 TwoCells in a row. 
If A.(B.C) is taken to mean horizontally composing (B.C) and then composing with A, (with the understanding that the order of composition is C, then B, then A), 
then Get\[Omega][GetDest[A],GetDest[B],GetDest[C]]*(A.(B.C)) == ((a.b).c))*Get\[Omega][GetSource[A],GetSource[B],GetSource[C]] with \[Omega]*(A.(B.C)) taken to mean vertical composition of \[Omega] with (A.(B.C)) 
(again, in the order (A.(B.C)) followed by \[Omega]. Note that there is a DIFFERENT \[Omega] on each side*)
Get\[Omega][ia_,ib_,ic_]:=Module[{a,b,c,lists,lists2,shift,g,GetReshuffleMatrix,GetReorderMatrix,\[Omega]PartI,\[Omega]PartII,f},
	If[GetCellType[ia]==0,a=GetIdentity[ia],a=ia];
	If[GetCellType[ib]==0,b=GetIdentity[ib],b=ib];
	If[GetCellType[ic]==0,c=GetIdentity[ic],c=ic];
	If[GetSource[b]!=GetDest[c]||GetSource[a]!=GetDest[b],
		Print["Error: OneCells not compatible for composition."],

		If[GetDest[c]==1, 
			GetIdentityTwoCell[OneCellComposition[OneCellComposition[a,b],c]]
			,
			f[u_]:=If[u!=0,IdentityMat[u],0];

			GetReshuffleMatrix[list_]:=Module[{ReshuffleMatrix,k,subMatrix,subList,m,i,col,row,aa,zerorows,zerocols,listfix},
			
				ReshuffleMatrix = {\[EmptySet],{0->0}};

				(* fix problem with Appending on GeneralizedDot in the function one level above this one *)
				listfix=list;
				listfix[[1]]=Append[{listfix[[1]]},listfix[[2]]];
				listfix=Delete[listfix,2];

				For[k=1,k<=Length[listfix],k++,
					subList=listfix[[k]];
					aa=subList[[1]];
					zerorows={};
					zerocols = {};

					If[aa==0||Total[subList[[2]]]==0, 
						subMatrix= {\[EmptySet],{0->0}},
						subMatrix=ConstantArray[0,{aa*Length[subList[[2]] ],aa*Length[subList[[2]] ]}];

						For[m=1, m<=Length[subList[[2]] ],m++,
				
							For[i=1,i<=aa,i++,
								col=(i-1)*Length[subList[[2]] ] +m;
								row = (m-1)*aa+i;
		
								If[subList[[2]][[m]]!=0,
									subMatrix[[row,col]]=subList[[2]][[m]],
									zerorows=Append[zerorows,{row}]; 
									zerocols=Append[zerocols,{col}] 
								];
							];
						];

						(*eliminate zeros*)
						subMatrix=Transpose[Delete[Transpose[Delete[subMatrix,zerorows]],zerocols]];
					];

					ReshuffleMatrix=DirectSum[ReshuffleMatrix,subMatrix];
				];

				If[Dim[ReshuffleMatrix]!={0,0},
					ReshuffleMatrix=ArrayFlatten[GeneralizedUnaryElementOperation[ReshuffleMatrix,f]]
				];

				ReshuffleMatrix
			];

			GetReorderMatrix[list_,shift_]:=Module[{i,j,num,ReorderMatrix,sum,zerorows,zerocols},
			
				sum=Total[list];

				If[sum==0,{\[EmptySet],{0->0}},
					ReorderMatrix=ConstantArray[0,{Length[list],Length[list]}];
					num=1;
					zerorows={};
					zerocols = {};

					If[list[[1]]!=0,
						ReorderMatrix[[1,1]]=list[[1]],
						zerorows=Append[zerorows,{1}];
						zerocols=Append[zerocols,{1}]
					];

					For[i=2,i<=Length[list],i++,
						num+=shift;

						If[num>Length[list], num=num-Length[list]+1];

						If[list[[num]]!=0,
							ReorderMatrix[[i,num]]=list[[num]],
							zerorows=Append[zerorows,{i}]; 
							zerocols=Append[zerocols,{num}]
						];

					];

					ReorderMatrix=Transpose[Delete[Transpose[Delete[ReorderMatrix,zerorows]],zerocols]];
					ArrayFlatten[GeneralizedUnaryElementOperation[ReorderMatrix,f]]
				]
			];

			lists=GeneralizedDot[OneToMat[a],GeneralizedDot[OneToMat[b],OneToMat[c],Composition[Flatten,List],Times],Append,List];
			\[Omega]PartI=(GeneralizedUnaryElementOperation[lists,GetReshuffleMatrix]//TwoCell);
			lists2=GeneralizedDot[OneToMat[a],GeneralizedDot[OneToMat[b],OneToMat[c],Composition[Flatten,List],Times],Composition[Flatten,List],Times];
			shift=GetSource[b];

			g[u_]:=GetReorderMatrix[u,shift];

			\[Omega]PartII=(GeneralizedUnaryElementOperation[lists2,g]//TwoCell);

			TwoCellVerticalComposition[\[Omega]PartII,\[Omega]PartI]
		]
	]
]

(* Get\[Tau] is used to generate \[Tau], the structural change of basis map used to correct for horizontally composing 2 pairs of TwoCells and 
then 2 monoidally combining them is not the same as 2 monoidally combining the two and then horizontally composing them. 
If (A.B)[x](C.D) is taken to mean horizontally composing (A.B) and (C.D) and then monoidally combining them, 
then Get\[Tau][GetDest[A],GetDest[B],GetDest[C],GetDest[D]]*(A.B)[x](C.D) == (A[x]C).(B[x]D)*Get\[Tau][GetSource[A],GetSource[B],GetSource[C],GetSource[D]] 
with the '*' operation taken to mean vertical composition.
Note that there is a DIFFERENT \[Tau] acting on each side that is only the same if TwoCells A,B,C, and D each individually have their source equal their destination*)
Get\[Tau][ia_,ib_,ic_,id_]:=Module[{a,b,c,d,lists1,lists2,lists3, lists4,lists5, lists6, Get\[Tau]Matrix},
	If[GetCellType[ia]==0,a=GetIdentity[ia],a=ia];
	If[GetCellType[ib]==0,b=GetIdentity[ib],b=ib];
	If[GetCellType[ic]==0,c=GetIdentity[ic],c=ic];
	If[GetCellType[id]==0,d=GetIdentity[id],d=id];
	If[GetSource[a]!=GetDest[b]||GetSource[c]!=GetDest[d],
		Print["Error: OneCells not compatible for composition."],

		Get\[Tau]Matrix[list3_,list4_]:=Module[{list1,list2,acTypeTerms,NumacTypeTerms,egTypeTerms,NumegTypeTerms,\[Tau]Matrix,Sub\[Tau]Matrix,i,j,k,specificacTerm,specificegTerm,row,col,zerorows,zerocols,f,specificAterm,specificCterm,specificEterm,specificGterm,aa,cc,ee,ReshuffleSubMat,ReshuffleMat,ReshuffleSubSubMat},
			list1=list3[[1]]*list3[[2]];
			list2=list4[[1]]*list4[[2]];

			NumacTypeTerms=Length[list1];
			NumegTypeTerms = Length[list2 ];

			\[Tau]Matrix={\[EmptySet],{0->0}};
			ReshuffleMat={\[EmptySet],{0->0}};

			If[Total[list2]!=0,
				For[i=1,i<=NumacTypeTerms,i++,
					specificacTerm=list1[[i]];
						
					If[specificacTerm!=0,
						Sub\[Tau]Matrix=ConstantArray[0,{specificacTerm*NumegTypeTerms,specificacTerm*NumegTypeTerms}];
						zerorows={};
						zerocols = {};
						ReshuffleSubMat={\[EmptySet],{0->0}};
	
						For[k=1,k<=NumegTypeTerms,k++,
							specificegTerm=list2[[k]];

							For[j=1,j<=specificacTerm,j++,
								col=(j-1)*NumegTypeTerms+k;
								row=(k-1)*specificacTerm+j;

								If[specificegTerm==0,
									zerorows=Append[zerorows,{row}];
									zerocols=Append[zerocols,{col}];,
								
									Sub\[Tau]Matrix[[row,col]]=specificegTerm
								];
							];

							(*deal with creating reshuffle matrix*)
							If[specificegTerm!=0,
								specificAterm =list3[[1]][[i]];
								specificCterm =list3[[2]][[i]];
								specificEterm =list4[[1]][[k]];
								specificGterm =list4[[2]][[k]];
								ReshuffleSubSubMat=ConstantArray[0,{specificCterm*specificEterm,specificCterm*specificEterm}];

								For[cc=1,cc<=specificCterm,cc++,
									For[ee=1,ee<=specificEterm,ee++,
										row=(ee-1)*specificCterm +cc;
										col=(cc-1)*specificEterm +ee;
										ReshuffleSubSubMat[[row,col]]=specificGterm;
									];
								];
									
								For[aa=1, aa<=specificAterm, aa++,
									ReshuffleSubMat=DirectSum[ReshuffleSubMat,ReshuffleSubSubMat];
								];
							]
							(*end reshuffle creationg. note, there are no zero rows or cols*)
						];
							
						Sub\[Tau]Matrix=Transpose[Delete[Transpose[Delete[Sub\[Tau]Matrix,zerorows]],zerocols]];
						\[Tau]Matrix=DirectSum[\[Tau]Matrix,Sub\[Tau]Matrix];
						ReshuffleMat=DirectSum[ReshuffleMat,ReshuffleSubMat];
					];
				];
			];

			f[u_]:=If[u!=0,IdentityMat[u],0];

			If[Dim[\[Tau]Matrix]!={0,0},
				\[Tau]Matrix=ArrayFlatten[GeneralizedUnaryElementOperation[\[Tau]Matrix,f]]
			];

			If[Dim[ReshuffleMat]!={0,0},
				ReshuffleMat=ArrayFlatten[GeneralizedUnaryElementOperation[ReshuffleMat,f]]
			];

			Dot0D[ReshuffleMat,\[Tau]Matrix]
		];

		(*lists1=GeneralizedDot[a//OneToMat,b//OneToMat,Composition[Flatten,List],Times];*)
		lists1=GeneralizedDot[a//OneToMat,IdentityMatrix[GetSource[a]],Composition[Flatten,List],Times];
		lists3=GeneralizedDot[IdentityMatrix[GetDest[b]],b//OneToMat,Composition[Flatten,List],Times];

		lists5=GeneralizedDot[lists1,lists3,Plus,List];
		(*lists2= GeneralizedDot[c//OneToMat,d//OneToMat,Composition[Flatten,List],Times];*)

		lists2=GeneralizedDot[c//OneToMat,IdentityMatrix[GetSource[c]],Composition[Flatten,List],Times];
		lists4=GeneralizedDot[IdentityMatrix[GetDest[d]],d//OneToMat,Composition[Flatten,List],Times];
		lists6=GeneralizedDot[lists2,lists4,Plus,List];
		GeneralizedTensorProduct[lists5,lists6,Get\[Tau]Matrix]//TwoCell
	]
]

(* Get\[Sigma] is used to generate effectively a TwoCell Swap, i.e. an interchange between a OneCellSwap and OneCells on each leg of the swap. 
It mediates between S(F[x]G) and (G[x]F)S, where the S's are swapping different objects. 
Thus, at the TwoCell level, if * means vertical composition and . means horizontal composition, and 
\[Mu]: F1->F2 and \[Nu]:G1->G2, then Get\[Sigma][F2,G2]*(idS_BD.(\[Mu][x]\[Nu]) ) == ( (\[Nu][x]\[Mu]).idS_BD )*Get\[Sigma][F1,G1]  *)
(*NOTE: Will not work if one of the input OneCells are OD*)
Get\[Sigma][iA_,iB_]:=Module[{A,B,g,ans1,ans2,template,f,SwapAB},
	(* Since 2Matrices are matrices of matrices, a swap is required for both the outer an inner matrices. 
	 The outer matrix swap is handled by the implicit idS that given by the Whiskering of the OneCell Swap gate present when a \[Sigma] is required. 
	 However, this shuffles the order of the outer matrices, and the \[Sigma] must be similarly swaped to be compatible.
	 Basically, since \[Sigma]'s source is a OneCell of the form S(A[x]B), where S is OneCellSwap, 
	 the first part (solved in ans1) fixes A[x]B and the second part is the expected Whiskering of S*)

	If[GetCellType[iA]==0,A=GetIdentity[iA],A=iA];
	If[GetCellType[iB]==0,B=GetIdentity[iB],B=iB];

	(*fix the inner matrix order problem (i.e. the actual purpose of sigma)*)
	g[a_,b_]:=OneToMat[GetOneCellSwap[a,b]];
	ans1=GeneralizedTensorProduct[A//OneToMat,B//OneToMat,g]//TwoCell;

	(* fix the outer matrix order to be compatible  *)
	SwapAB=GetOneCellSwap[GetDest[A],GetDest[B]];
	ans2=GetIdentityTwoCell[SwapAB];
	
	TwoCellHorizontalComposition[ans2,ans1]
]

(*Redefitions for convenience*)
\[Omega][a_,b_,c_]:=Get\[Omega][a,b,c]
\[Tau][a_,b_,c_,d_]:=Get\[Tau][a,b,c,d]
\[Sigma][a_,b_]:=Get\[Sigma][a,b] 


(*Display Functions*)

(*Display matrices of matrices (i.e. 2-Cells without the 1-Cell part ...the equivalent of what a matrix is to a 1-Cell) *)
TwoMatrixForm[F_]:=GeneralizedUnaryElementOperation[F,MatrixForm]//MatrixForm

(* Display 1-Cells in easy readable form *)
OneCellForm[f_]:=MatrixForm/@f

(* Display 2-Cells in easy readable form *)
TwoCellForm[F_]:=Module[{MatrixFormFixZero},

MatrixFormFixZero[a_]:=If[Is0D[a],Subscript[0,GetSource[a]->GetSource[a]],MatrixForm[a]];

{GeneralizedUnaryElementOperation[F[[1]],MatrixFormFixZero]//MatrixForm,OneCellForm/@F[[2]][[1]]}
]


(*Overloaded operations that work for 1-Cells and 2-Cells*)

(*Computes either 1-cell composition or 2-cell horizontal composition, depending on the cell type of the arguments*)
(*SetAttributes[SmallCircle,{Flat,OneIdentity}]*)
SmallCircle[A_,B_]:=Module[{TypeA,TypeB,maxtype,nCellA,nCellB},
	TypeA=GetCellType[A];
	TypeB=GetCellType[B];
	maxtype=Max[TypeA,TypeB];
	If[maxtype==0,Print["Cannot horizontally compose 0-cells."];Return[];];
	If[(maxtype!=1)&&(maxtype!=2),Print["Arguments are not 1-cells or 2-cells."]; Return[];];
	nCellA=A; nCellB=B;
	While[GetCellType[nCellA]<maxtype,nCellA=GetIdentity[nCellA]];
	While[GetCellType[nCellB]<maxtype,nCellB=GetIdentity[nCellB]];
	If[maxtype==1,Return[OneCellComposition[nCellA,nCellB]]];
	If[maxtype==2,Return[TwoCellHorizontalComposition[nCellA,nCellB]]];
]
(* ???
SmallCircle[A_,B_,C_]:=Module[{AB,TypeAB,TypeC},

	AB=SmallCircle[A,B];
	TypeAB = GetCellType[AB];
	TypeC = GetCellType[C];

	If[TypeAB!=TypeC,
		Print["Cell types do not match."],
		If[TypeAB==1,
			OneCellComposition[AB,C],
			If[TypeAB==2,
				TwoCellVerticalComposition[AB,C],
				Print["Arguments are not 1-cells or 2-cells."]
			]
		]
	]
] *)

(*A_\[CircleDot]B_ Computes either 1-cell parallel combination or 2-cell parallel combination, in 2Vect, depending on the cell type of the arguments. Note: this is usually represented by the [x] operation.*)
(*SetAttributes[CircleDot,{Flat,OneIdentity}]*)
CircleDot[A_,B_]:=Module[{TypeA,TypeB,nCellA,nCellB,maxtype},
	TypeA=GetCellType[A];
	TypeB=GetCellType[B];
	maxtype=Max[TypeA,TypeB];
	nCellA=A; nCellB=B;
	While[GetCellType[nCellA]<maxtype,nCellA=GetIdentity[nCellA]];
	While[GetCellType[nCellB]<maxtype,nCellB=GetIdentity[nCellB]];
	If[maxtype==0,Return[A B]];
	If[maxtype==1,Return[OneCellTimes[nCellA,nCellB]]];
	If[maxtype==2,Return[TwoCellTimes[nCellA,nCellB]]];
]
(* ???
CircleDot[A_,B_,C_]:=Module[{TypeAB,TypeC,AB},

	AB=CircleDot[A,B];

	TypeAB=GetCellType[AB];
	TypeC=GetCellType[C];

	If[TypeAB!=TypeC,
		Print["Cell types do not match."],
		If[TypeAB==1,
			OneCellTimes[AB,C],
			If[TypeAB==2,
				TwoCellTimes[AB,C],
				Print["Arguments are not 1-cells or 2-cells."]
			]
		]
	]
] *)

(* A_\[CenterDot]B_ Computes 2-cell vertical composition. Arguments must be 2-cells. *)
SetAttributes[CenterDot,{Flat,OneIdentity}]
CenterDot[A_,B_]:=Module[{TypeA,TypeB,ArgA,ArgB},
	TypeA=GetCellType[A];
	TypeB=GetCellType[B];
	If[(TypeA!=2)||(TypeB!=2),Print["Arguments are not 2-cells."]; Return[];];
	Return[TwoCellVerticalComposition[A,B]];	
]

(* Returns either the identity 1-cell or the identity 2-cell, if the argument is a 0-cell or 1-cell respectively *)
id[A_]:=Module[{TypeA},
	TypeA=GetCellType[A];
	If[TypeA==0,
		GetIdentityOneCell[A],
		If[TypeA==1,
			GetIdentityTwoCell[A],
			Print["Arguments are not 1-cells or 2-cells."]
		]
	]
]

(* Returns either the inverse 1-cell or the inverse 2-cell, if the argument is a 1-cell or 2-cell respectively *)
inv[A_]:=Module[{TypeA},
	TypeA=GetCellType[A];
	If[TypeA==1,
		OneCellInverse[A],
		If[TypeA==2,
			TwoCellInverse[A],
			Print["Arguments are not 1-cells or 2-cells."]
		]
	]
]

(*Vertically composes a list of 2-cells. The first element of the list is the first 2-cell applied, 
which is usually written as the right-most 2-cell (i.e. if A and B are 2-cells and we want to compute A\[SmallCircle]B, 
then we would enter {B,A} as the argument to this function)*)
Compose2Cells[list_]:=Module[{n,i,ans},

	If[Depth[list]<=1,
		Print["Argument is not a list."],

		n=Length[list];

		If[GetCellType[list[[1]]]!=2,
			Print["Element 1 is not a 2-cell."],
			ans=list[[1]];

			For[i=2,i<=n,i++,
				If[GetCellType[list[[i]]]!=2,
					Print["Element ",i," is not a 2-cell."],
					ans=list[[i]]\[SmallCircle]ans
				]
			];
			ans
		]
	]
]


End[ ]

EndPackage[ ]

